@@ -41,6 +41,7 @@ from ..entity_base import class_by_attr
 from ..helpers import shrink
 from ..protocol import Address, Message
 from ..protocol.command import Command
+from ..protocol.const import SZ_BINDINGS
 from ..protocol.ramses import CODES_OF_HVAC_DOMAIN_ONLY, HVAC_KLASS_BY_VC_PAIR
 from ..schemas import SCH_VCS, SZ_REMOTES, SZ_SENSORS
 from .base import BatteryState, Device, DeviceHvac, Fakeable
@@ -83,7 +84,9 @@ class CarbonDioxide(Fakeable, HvacSensorBase):  # 1298
         # .I --- 29:181813 32:155617 --:------ 1FC9 001 00
 
         def callback(msg):
-            self.set_parent(msg.src, child_id=msg.payload[0][0], is_sensor=True)
+            """Use the accept pkt to determine the zone/domain id."""
+            _ = msg.payload[SZ_BINDINGS][0][0]
+            # self.set_parent(msg.src, child_id=child_id, is_sensor=True)
 
         super()._bind()
         self._bind_request((Code._1298, Code._31E0), callback=callback)
@@ -117,7 +120,9 @@ class IndoorHumidity(Fakeable, HvacSensorBase):  # 12A0
         # .I ---
 
         def callback(msg):
-            self.set_parent(msg.src, child_id=msg.payload[0][0], is_sensor=True)
+            """Use the accept pkt to determine the zone/domain id."""
+            _ = msg.payload[SZ_BINDINGS][0][0]
+            # self.set_parent(msg.src, child_id=child_id, is_sensor=True)
 
         super()._bind()
         self._bind_request((Code._12A0, Code._31E0), callback=callback)
@@ -151,7 +156,9 @@ class PresenceDetect(Fakeable, HvacSensorBase):  # 2E10
         # .I --- 37:154011 28:126620 --:------ 1FC9 001 00                                                                            # CO2, incl. integrated control, PIR
 
         def callback(msg):
-            self.set_parent(msg.src, child_id=msg.payload[0][0], is_sensor=True)
+            """Use the accept pkt to determine the zone/domain id."""
+            _ = msg.payload[SZ_BINDINGS][0][0]
+            # self.set_parent(msg.src, child_id=child_id, is_sensor=True)
 
         super()._bind()
         self._bind_request((Code._2E10, Code._31E0), callback=callback)
@@ -179,10 +186,10 @@ class PresenceDetect(Fakeable, HvacSensorBase):  # 2E10
 class FilterChange(DeviceHvac):  # FAN: 10D0
     """The filter state sensor (10D0)."""
 
-    def _setup_discovery_tasks(self) -> None:
-        super()._setup_discovery_tasks()
+    def _setup_discovery_cmds(self) -> None:
+        super()._setup_discovery_cmds()
 
-        self._add_discovery_task(
+        self._add_discovery_cmd(
             Command.from_attrs(RQ, self.id, Code._10D0, "00"), 60 * 60 * 24, delay=30
         )
 
@@ -251,8 +258,10 @@ class HvacRemote(BatteryState, Fakeable, HvacRemoteBase):  # REM: I/22F[138]
         # .W --- 32:155617 37:155617 --:------ 1FC9 012 0031D9825FE10031DA825FE1
         # .I --- 37:155617 32:155617 --:------ 1FC9 001 00
 
-        def callback(msg):
-            self.set_parent(msg.src, child_id=msg.payload[0][0], is_sensor=True)
+        def callback(msg):  # TODO: set_parent()
+            """Use the accept pkt to determine the zone/domain id."""
+            _ = msg.payload[SZ_BINDINGS][0][0]
+            # self.set_parent(msg.src, child_id=child_id, is_sensor=True)
 
         super()._bind()
         self._bind_request((Code._22F1, Code._22F3), callback=callback)
@@ -321,11 +330,11 @@ class HvacVentilator(FilterChange):  # FAN: RP/31DA, I/31D[9A]
         for dev_id in schema.get(SZ_SENSORS, {}).keys():
             self._gwy.get_device(self._gwy, dev_id)
 
-    def _setup_discovery_tasks(self) -> None:
-        super()._setup_discovery_tasks()
+    def _setup_discovery_cmds(self) -> None:
+        super()._setup_discovery_cmds()
 
         # RP --- 32:155617 18:005904 --:------ 22F1 003 000207
-        self._add_discovery_task(
+        self._add_discovery_cmd(
             Command.from_attrs(RQ, self.id, Code._22F1, "00"), 60 * 60 * 24, delay=15
         )  # to learn scheme: orcon/itho/other (04/07/0?)
 
@@ -338,12 +347,12 @@ class HvacVentilator(FilterChange):  # FAN: RP/31DA, I/31D[9A]
             Code._22F4,
             Code._22F8,
         ):
-            self._add_discovery_task(
+            self._add_discovery_cmd(
                 Command.from_attrs(RQ, self.id, code, "00"), 60 * 30, delay=15
             )
 
         for code in (Code._313E, Code._3222):
-            self._add_discovery_task(
+            self._add_discovery_cmd(
                 Command.from_attrs(RQ, self.id, code, "00"), 60 * 30, delay=30
             )
 
