@@ -15,9 +15,10 @@ from datetime import datetime as dt
 from datetime import timedelta as td
 from inspect import getmembers, isclass
 from sys import modules
-from typing import Any
+from typing import TYPE_CHECKING, Any
 
 from .const import (
+    DEV_TYPE_MAP,
     SZ_ACTUATORS,
     SZ_DEVICE_ID,
     SZ_DOMAIN_ID,
@@ -27,8 +28,9 @@ from .const import (
     SZ_ZONE_IDX,
     __dev_mode__,
 )
-from .protocol import CorruptStateError, Message
+from .protocol import CorruptStateError
 from .protocol.frame import _CodeT, _DeviceIdT, _HeaderT, _VerbT
+from .protocol.opentherm import OPENTHERM_MESSAGES
 from .protocol.ramses import CODES_SCHEMA
 from .protocol.transport import PacketProtocolPort
 from .schemas import SZ_CIRCUITS
@@ -46,6 +48,9 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     Code,
 )
 
+if TYPE_CHECKING:
+    from .protocol import Command, Message, Packet
+
 
 _QOS_TX_LIMIT = 12  # TODO: needs work
 
@@ -222,23 +227,64 @@ class Discovery(MessageDB):
     def __init__(self, gwy, *args, **kwargs) -> None:
         super().__init__(gwy, *args, **kwargs)
 
-        self._disc_tasks: dict[_HeaderT, dict] = None  # type: ignore[assignment]
-        self._disc_tasks_poller = None
+        self._discovery_cmds: dict[_HeaderT, dict] = None  # type: ignore[assignment]
+        self._discovery_poller = None
+
+        self._supported_cmds: dict[str, None | bool] = {}
+        self._supported_cmds_ctx: dict[str, None | bool] = {}
 
         if not gwy.config.disable_discovery and isinstance(
             gwy.pkt_protocol, PacketProtocolPort
         ):  # TODO: here, or in get_xxx()?
             # gwy._loop.call_soon_threadsafe(
-            #     gwy._loop.call_later, random(0.5, 1.5), self._start_discovery_poller
+            #     gwy._loop.call_later, random(0.5, 1.5), self.start_discovery_poller
             # )
             gwy._loop.call_soon(self._start_discovery_poller)
 
-    def _setup_discovery_tasks(self) -> None:
+    @property  # TODO: needs tidy up
+    def discovery_cmds(self) -> dict:
+        """Return the pollable commands."""
+        if self._discovery_cmds is None:
+            self._discovery_cmds = {}
+            self._setup_discovery_cmds()
+        return self._discovery_cmds
+
+    @property
+    def supported_cmds(self) -> dict:
+        """Return the current list of pollable command codes."""
+        return {
+            code: (CODES_SCHEMA[code][SZ_NAME] if code in CODES_SCHEMA else None)
+            for code in sorted(self._msgz)
+            if self._msgz[code].get(RP) and self.is_pollable_cmd(code)
+        }
+
+    @property
+    def supported_cmds_ot(self) -> dict:
+        """Return the current list of pollable OT msg_ids."""
+        return {
+            msg_id: OPENTHERM_MESSAGES[int(msg_id, 16)].get("var")
+            for msg_id in sorted(self._msgz[Code._3220].get(RP, []))
+            if self.is_pollable_cmd(Code._3220, ctx=msg_id)
+        }
+
+    def is_pollable_cmd(self, code, ctx=None) -> bool:
+        """Return True if the code|ctx pair is not deprecated."""
+
+        if ctx is None:
+            supported_cmds = self._supported_cmds
+            idx = code
+        else:
+            supported_cmds = self._supported_cmds_ctx
+            idx = f"{code}|{ctx}"
+
+        return supported_cmds.get(idx, None) is not False
+
+    def _setup_discovery_cmds(self) -> None:
         raise NotImplementedError
 
-    def _add_discovery_task(
-        self, cmd, interval, *, timeout: float = None, delay: float = 0
-    ):
+    def _add_discovery_cmd(
+        self, cmd: Command, interval, *, timeout: float = None, delay: float = 0
+    ) -> None:
         """Schedule a command to run periodically.
 
         Both `timeout` and `delay` are in seconds.
@@ -250,7 +296,7 @@ class Discovery(MessageDB):
             _LOGGER.warning(f"cmd({cmd}): invalid (null) header not added to discovery")
             return
 
-        if cmd.rx_header in self._disc_tasks:
+        if cmd.rx_header in self.discovery_cmds:
             _LOGGER.info(f"cmd({cmd}): duplicate header not added to discovery")
             return
 
@@ -260,7 +306,7 @@ class Discovery(MessageDB):
             timeout or (cmd._qos.retry_limit + 1) * cmd._qos.rx_timeout.total_seconds()
         )
 
-        self._disc_tasks[cmd.rx_header] = {
+        self.discovery_cmds[cmd.rx_header] = {
             "command": cmd,
             "interval": td(seconds=max(interval, self.MAX_CYCLE_SECS)),
             "last_msg": None,
@@ -270,63 +316,69 @@ class Discovery(MessageDB):
         }
 
     def _start_discovery_poller(self) -> None:
-        if self._disc_tasks is None:
-            self._disc_tasks = {}
-            self._setup_discovery_tasks()
+        if not self._discovery_poller or self._discovery_poller.done():
+            self._discovery_poller = self._gwy.add_task(self._poll_discovery_cmds)
+            self._discovery_poller.set_name(f"{self.id}_discovery_poller")
+            pass
 
-        if not self._disc_tasks_poller or self._disc_tasks_poller.done():
-            self._disc_tasks_poller = self._gwy.add_task(self._poll_discovery_tasks)
+    async def _stop_discovery_poller(self) -> None:
+        if self._discovery_poller and not self._discovery_poller.done():
+            self._discovery_poller.cancel()
+            try:
+                await self._discovery_poller
+            except asyncio.CancelledError:
+                pass
 
-    def _stop_discovery_poller(self) -> None:
-        if self._disc_tasks_poller and not self._disc_tasks_poller.done():
-            self._disc_tasks_poller.cancel()
-
-    async def _poll_discovery_tasks(self) -> None:
+    async def _poll_discovery_cmds(self) -> None:
         """Send any outstanding commands that are past due.
 
         If a relevant message was received recently enough, reschedule the corresponding
         command for later.
         """
 
-        def find_newer_msg(hdr: _HeaderT, task: dict) -> None | Message:
+        while True:
+            if self._gwy.config.disable_discovery:  # TODO: remove?
+                await asyncio.sleep(self.MIN_CYCLE_SECS)
+                continue
+
+            await self.discover()
+
+            if self.discovery_cmds:
+                next_due = min(t["next_due"] for t in self.discovery_cmds.values())
+                delay = max((next_due - dt.now()).total_seconds(), self.MIN_CYCLE_SECS)
+            else:
+                delay = self.MAX_CYCLE_SECS
+
+            await asyncio.sleep(min(delay, self.MAX_CYCLE_SECS))
+
+    async def discover(self) -> None:
+        def find_latest_msg(hdr: _HeaderT, task: dict) -> None | Message:
+            """Return the latest message for a header from any source (not just RPs)."""
             msgs: list[Message] = [
                 m
                 for m in [self._get_msg_by_hdr(hdr[:5] + v + hdr[7:]) for v in (I_, RP)]
                 if m is not None
             ]
 
-            if (
-                msgs
-                and (msg := max(msgs))
-                and msg.dtm > task["next_due"] - task["interval"]
-            ):
-                return msg
+            try:
+                if task["command"].code in (Code._000A, Code._30C9):
+                    msgs += [self.tcs._msgz[task["command"].code][I_][True]]
+            except KeyError:
+                pass
 
-            # has the controller sent an array recently - use that instead?
-            for code in (
-                Code._000A,
-                Code._30C9,
-            ):  # can't use 2309 (no mode) instead of 2349
-                try:
-                    if task["command"].code == code and (
-                        msg := self.tcs._msgz[code][I_][True]
-                    ):
-                        return msg
-                except KeyError:
-                    pass
+            return (msgs and max(msgs)) or None
 
-            return None
-
-        def interval(hdr: _HeaderT, failures: int) -> td:
-            """Adjust the ineterval - backoff if any failures."""
+        def backoff(hdr: _HeaderT, failures: int) -> td:
+            """Backoff the interval if there are/were any failures."""
 
             if failures > 5:
-                secs = 60 * 60 * 24
-                _LOGGER.warning(f"No response for task({hdr}): throttling to 1/24h")
+                secs = 60 * 60 * 6
+                _LOGGER.warning(f"No response for task({hdr}): throttling to 1/6h")
             elif failures > 2:
-                _LOGGER.debug(f"No response for task({hdr}): throttling")
+                _LOGGER.debug(f"No response for task({hdr}): retrying in 30s")
                 secs = self.MAX_CYCLE_SECS
             else:
+                # _LOGGER.info(f"No response for task({hdr}): retrying in 3s")
                 secs = self.MIN_CYCLE_SECS
 
             return td(seconds=secs)
@@ -354,38 +406,71 @@ class Discovery(MessageDB):
 
             return None
 
-        while True:
-            if self._gwy.config.disable_discovery:
-                await asyncio.sleep(self.MIN_CYCLE_SECS)
+        for hdr, task in self.discovery_cmds.items():
+            dt_now = dt.now()
+
+            if (msg := find_latest_msg(hdr, task)) and (
+                task["next_due"] < msg.dtm + task["interval"]
+            ):  # if a newer message is available, take it
+                task["failures"] = 0  # only if task["last_msg"].verb == RP?
+                task["last_msg"] = msg
+                task["next_due"] = msg.dtm + task["interval"]
+
+            if task["next_due"] > dt_now:
+                continue  # if (most recent) last_msg is is not yet due...
+
+            # since we may do I/O, check if the code|msg_id is deprecated
+            task["next_due"] = dt_now + task["interval"]  # might undeprecate later
+
+            if not self.is_pollable_cmd(task["command"].code):
+                continue
+            if not self.is_pollable_cmd(
+                task["command"].code, ctx=task["command"].payload[4:6]
+            ):  # only for Code._3220
                 continue
 
-            for hdr, task in self._disc_tasks.items():
-                dt_now = dt.now()
+            # we'll have to do I/O...
+            task["next_due"] = dt_now + backoff(hdr, task["failures"])  # JIC
 
-                if msg := find_newer_msg(hdr, task):
-                    task["last_msg"] = msg
-                elif task["next_due"] <= dt_now:
-                    task["next_due"] = dt_now + task["interval"]
-                    task["last_msg"] = await send_disc_task(hdr, task)
-                else:
-                    continue
-
-                if task["last_msg"]:
-                    task["failures"] = 0  # only if task["last_msg"].verb == RP?
-                    task["next_due"] = task["last_msg"].dtm + task["interval"]
-                else:
-                    task["failures"] += 1
-                    task["next_due"] = dt_now + interval(hdr, task["failures"])
-
-            if self._disc_tasks:
-                seconds = (
-                    min(t["next_due"] for t in self._disc_tasks.values()) - dt.now()
-                ).total_seconds()
-                await asyncio.sleep(
-                    min(max(seconds, self.MIN_CYCLE_SECS), self.MAX_CYCLE_SECS)
-                )
+            if msg := await send_disc_task(hdr, task):  # TODO: OK 4 some exceptions
+                task["failures"] = 0  # only if task["last_msg"].verb == RP?
+                task["last_msg"] = msg
+                task["next_due"] = msg.dtm + task["interval"]
             else:
-                await asyncio.sleep(self.MAX_CYCLE_SECS)
+                task["failures"] += 1
+                task["last_msg"] = None
+                task["next_due"] = dt_now + backoff(hdr, task["failures"])
+
+    def deprecate_cmd(self, pkt: Packet, ctx: str = None, reset: bool = False) -> None:
+        """If a code|ctx is deprecated twice, stop polling for it."""
+
+        def deprecate(supported_dict, idx):
+            if idx not in supported_dict:
+                supported_dict[idx] = None
+            elif supported_dict[idx] is None:
+                _LOGGER.warning(
+                    f"{pkt} < Polling now deprecated for code|ctx={idx}: "
+                    "it appears to be unsupported"
+                )
+                supported_dict[idx] = False
+
+        def reinstate(supported_dict, idx):
+            if self.is_pollable_cmd(idx, None) is False:
+                _LOGGER.warning(
+                    f"{pkt} < Polling now reinstated for code|ctx={idx}: "
+                    "it now appears supported"
+                )
+            if idx in supported_dict:
+                supported_dict.pop(idx)
+
+        if ctx is None:
+            supported_dict = self._supported_cmds
+            idx = pkt.code
+        else:
+            supported_dict = self._supported_cmds_ctx
+            idx = f"{pkt.code}|{ctx}"  # msg._pkt._ctx
+
+        (reinstate if reset else deprecate)(supported_dict, idx)
 
 
 class Entity(Discovery):
@@ -408,7 +493,9 @@ class Entity(Discovery):
     def __repr__(self) -> str:
         return f"{self.id} ({self._SLUG})"
 
-    def _qos_function(self, pkt, reset=False) -> None:
+    def deprecate(self, pkt, reset=False) -> None:
+        """If an entity is deprecated enough times, stop sending to it."""
+
         if reset:
             self._qos_tx_count = 0
             return
@@ -429,7 +516,7 @@ class Entity(Discovery):
             self._gwy.pkt_protocol is None
             or msg.src.id != self._gwy.pkt_protocol._hgi80.get(SZ_DEVICE_ID)
         ):
-            self._qos_function(msg._pkt, reset=True)
+            self.deprecate(msg._pkt, reset=True)
 
     def _make_cmd(self, code, dest_id, payload="00", verb=RQ, **kwargs) -> None:
         self._send_cmd(self._gwy.create_cmd(verb, dest_id, code, payload, **kwargs))
@@ -440,7 +527,7 @@ class Entity(Discovery):
             return None
 
         if self._qos_tx_count > _QOS_TX_LIMIT:
-            _LOGGER.info(f"{cmd} < Sending is deprecated for {self}")
+            _LOGGER.info(f"{cmd} < Sending now deprecated for {self}")
             return None
 
         cmd._source_entity = self
@@ -460,7 +547,7 @@ class Entity(Discovery):
         return {"_sent": list(codes.keys())}
 
 
-def _delete_msg(msg) -> None:
+def _delete_msg(msg: Message) -> None:
     """Remove the msg from all state databases."""
 
     entities = [msg.src]
@@ -694,9 +781,15 @@ class Child(Entity):  # A Zone, Device or a UfhCircuit
             if SZ_ZONE_IDX not in msg.payload:
                 return
 
-            if msg.code in (Code._1060, Code._12B0, Code._2309, Code._3150):  # not 30C9
+            # the follwing is a mess - may just be better off deprecating it
+            if self.type in DEV_TYPE_MAP.HEAT_ZONE_ACTUATORS:
                 self.set_parent(msg.dst, child_id=msg.payload[SZ_ZONE_IDX])
 
+            elif self.type in DEV_TYPE_MAP.THM_DEVICES:
+                self.set_parent(
+                    msg.dst, child_id=msg.payload[SZ_ZONE_IDX], is_sensor=True
+                )
+
         super()._handle_msg(msg)
 
         if not self._gwy.config.enable_eavesdrop or (
@@ -705,7 +798,7 @@ class Child(Entity):  # A Zone, Device or a UfhCircuit
         ):
             return
 
-        if True or not self._parent or not self._child_id:  # BUG:
+        if not self._parent or not self._child_id:
             eavesdrop_parent_zone()
 
     def _get_parent(
