@@ -19,13 +19,13 @@ from typing import Callable, Optional, TextIO
 
 from ramses_rf.device import DeviceHeat, DeviceHvac, Fakeable
 from ramses_rf.protocol.frame import _CodeT, _DeviceIdT, _PayloadT, _VerbT
-from ramses_rf.protocol.protocol import MessageProtocol, MessageTransport
-from ramses_rf.protocol.transport import PacketProtocolBase
+from ramses_rf.protocol.protocol import _MessageProtocolT, _MessageTransportT
+from ramses_rf.protocol.transport import _PacketProtocolT, _PacketTransportT
 
 from .const import DONT_CREATE_MESSAGES, SZ_DEVICE_ID, SZ_DEVICES, __dev_mode__
 from .device import Device, device_factory
+from .dispatcher import Message, process_msg
 from .helpers import schedule_task, shrink
-from .processor import Message, process_msg
 from .protocol import (
     SZ_POLLER_TASK,
     Address,
@@ -105,17 +105,25 @@ class Engine:
 
         self.config = SimpleNamespace()  # **SCH_CONFIG_GATEWAY({}))
 
-        self.msg_protocol: MessageProtocol = None  # type: ignore[assignment]
-        self.msg_transport: MessageTransport = None  # type: ignore[assignment]
-        self.pkt_protocol: PacketProtocolBase = None  # type: ignore[assignment]
-        self.pkt_transport: asyncio.Transport = None  # type: ignore[assignment]
+        self.msg_protocol: _MessageProtocolT = None  # type: ignore[assignment]
+        self.msg_transport: _MessageTransportT = None  # type: ignore[assignment]
+        self.pkt_protocol: _PacketProtocolT = None  # type: ignore[assignment]
+        self.pkt_transport: _PacketTransportT = None  # type: ignore[assignment]
 
         self._engine_lock = Lock()
         self._engine_state: None | tuple[None | Callable, tuple] = None
 
     def __str__(self) -> str:
-        hgi_id = self.pkt_protocol._hgi80[SZ_DEVICE_ID] if self.pkt_protocol else ""
-        return (hgi_id or HGI_DEV_ADDR.id) + f" ({self.ser_name})"
+        if self.pkt_protocol and self.pkt_protocol._hgi80[SZ_DEVICE_ID]:
+            return f"{self.pkt_protocol._hgi80[SZ_DEVICE_ID]} ({self.ser_name})"
+        return f"{HGI_DEV_ADDR.id} ({self.ser_name})"
+
+    @property
+    def hgi(self) -> None | Device:
+        """Return the active HGI80-compatible gateway device, if known."""
+        if self.pkt_protocol and self.pkt_protocol._hgi80[SZ_DEVICE_ID]:
+            return self.device_by_id.get(self.pkt_protocol._hgi80[SZ_DEVICE_ID])
+        return None
 
     def _setup_event_handlers(self) -> None:  # HACK: for dev/test only
         def handle_exception(loop, context):
@@ -134,8 +142,17 @@ class Engine:
         # return dt.now()
         return self.pkt_protocol._dt_now() if self.pkt_protocol else dt.now()
 
-    def create_client(self, msg_handler: Callable) -> tuple:
+    def create_client(
+        self,
+        msg_handler: Callable[[Message, Optional[Message]], None],
+        # msg_filter: Callable[[Message], bool] | None = None,
+    ) -> tuple[_MessageProtocolT, _MessageTransportT]:
         """Create a client protocol for the RAMSES-II message transport."""
+
+        # The optional filter will return True if the message is to be handled.
+        # """  # TODO
+        # if msg_filter is not None and not is_callback(msg_filter):
+        #     raise TypeError(f"Msg filter {msg_filter} is not a callback")
         return self._create_msg_stack(msg_handler)
 
     async def start(self) -> None:
@@ -248,9 +265,7 @@ class Engine:
         """Make a command addressed to device_id."""
         return Command.from_attrs(verb, device_id, code, payload, **kwargs)
 
-    def send_cmd(
-        self, cmd: Command, callback: Callable = None, **kwargs
-    ) -> futures.Future[Optional[Message]]:
+    def send_cmd(self, cmd: Command, callback: Callable = None, **kwargs):  # FIXME
         """Send a command with the option to return any response message via callback.
 
         Response packets, if any (an RP/I will follow an RQ/W), and have the same code.
@@ -260,12 +275,15 @@ class Engine:
         if not self.msg_protocol:
             raise RuntimeError("there is no message protocol")
 
-        return asyncio.run_coroutine_threadsafe(
-            self.msg_protocol.send_data(cmd, callback=callback, **kwargs),
-            self._loop,
-        )
+        # self._loop.call_soon_threadsafe(
+        #     self.msg_protocol.send_data(cmd, callback=callback, **kwargs)
+        # )
+        coro = self.msg_protocol.send_data(cmd, callback=callback, **kwargs)
+        fut: futures.Future = asyncio.run_coroutine_threadsafe(coro, self._loop)
+        # fut: asyncio.Future = asyncio.wrap_future(fut)
+        return fut
 
-    async def async_send_cmd(self, cmd: Command, **kwargs) -> None | Message:
+    async def async_send_cmd(self, cmd: Command, **kwargs) -> None | Message:  # FIXME
         """Send a command with the option to not wait for a response message.
 
         Response packets, if any, follow an RQ/W (as an RP/I), and have the same code.
@@ -408,16 +426,31 @@ class Gateway(Engine):
         elif start_discovery:  # source of packets is a serial port
             initiate_discovery(self.devices, self.systems)
 
-    async def stop(self) -> None:
+    async def stop(self) -> None:  # FIXME: a mess
         """Cancel all outstanding tasks."""
         # if self._engine_state is None:
         #     self._pause()
 
+        # (t for t in self._tasks if not isinstance(t, asyncio.Task))
+
         if [t.cancel() for t in self._tasks if not t.done()]:
-            try:  # TODO: except asyncio.CancelledError:
-                await asyncio.gather(*self._tasks)
-            except TypeError:  # HACK
+            await asyncio.gather(*(t for t in self._tasks if asyncio.isfuture(t)))
+
+        # this doesn't work...
+        for t in [t for t in self._tasks if isinstance(t, futures.Future)]:
+            try:
+                await asyncio.wrap_future(t, loop=self._loop)
+            except asyncio.CancelledError:
                 pass
+
+        # this doesn't work either
+        # for device in self.devices:
+        #     await device._stop_discovery_poller()
+        # for system in self.systems:
+        #     await system._stop_discovery_poller()
+        #     for zone in system.zones:
+        #         await zone._stop_discovery_poller()
+
         await super().stop()
 
     def _pause(self, *args, clear_state: bool = False) -> None:
@@ -463,13 +496,18 @@ class Gateway(Engine):
         return result
 
     def _get_state(self, include_expired: bool = False) -> tuple[dict, dict]:
-        def wanted_msg(msg: Message) -> bool:
-            # 313F will usu. be expired, but will be useful for back-back restarts
+        def wanted_msg(msg: Message, include_expired: bool = False) -> bool:
             if msg.code == Code._313F:
-                return True
+                return msg.verb in (I_, RP)  # usu. expired, useful 4 back-back restarts
+            if msg._expired and not include_expired:
+                return False
+            if msg.code == Code._0404:
+                return msg.verb in (I_, W_) and msg._pkt._len > 7
+            if msg.verb in (W_, RQ):
+                return False
             # if msg.code == Code._1FC9 and msg.verb != RP:
             #     return True
-            return not msg._expired
+            return include_expired or not msg._expired
 
         msgs = [m for device in self.devices for m in device._msg_db]
 
@@ -482,7 +520,7 @@ class Gateway(Engine):
         pkts = {
             f"{repr(msg._pkt)[:26]}": f"{repr(msg._pkt)[27:]}"
             for msg in msgs
-            if msg.verb in (I_, RP) and include_expired or wanted_msg(msg)
+            if wanted_msg(msg, include_expired=include_expired)
         }  # BUG: assumes pkts have unique dtms
 
         return self.schema, dict(sorted(pkts.items()))
@@ -536,12 +574,11 @@ class Gateway(Engine):
         def check_filter_lists(dev_id: _DeviceIdT) -> None:  # may: LookupError
             """Raise an LookupError if a device_id is filtered out by a list."""
 
-            if dev_id in self._unwanted:
+            if dev_id in self._unwanted:  # TODO: shoudln't invalidate a msg
                 raise LookupError(f"Can't create {dev_id}: it is unwanted or invalid")
 
             if self.config.enforce_known_list and (
-                dev_id not in self._include
-                and dev_id != self.pkt_protocol._hgi80[SZ_DEVICE_ID]
+                dev_id not in self._include and dev_id != getattr(self.hgi, "id", None)
             ):
                 self._unwanted.append(dev_id)
                 raise LookupError(
@@ -582,13 +619,6 @@ class Gateway(Engine):
 
         return dev
 
-    @property
-    def hgi(self) -> Optional[Device]:
-        """Return the HGI80-compatible gateway device."""
-        if self.pkt_protocol and self.pkt_protocol._hgi80[SZ_DEVICE_ID]:
-            return self.device_by_id.get(self.pkt_protocol._hgi80[SZ_DEVICE_ID])
-        return None
-
     @property
     def tcs(self) -> Optional[System]:
         """Return the primary TCS, if any."""
@@ -687,7 +717,7 @@ class Gateway(Engine):
     def status(self) -> dict:
         return {SZ_DEVICES: {d.id: d.status for d in sorted(self.devices)}}
 
-    def send_cmd(
+    def send_cmd(  # FIXME
         self, cmd: Command, callback: Callable = None, **kwargs
     ) -> futures.Future:
         """Send a command with the option to return any response via callback."""
@@ -730,9 +760,9 @@ class Gateway(Engine):
             return dev._make_fake(bind=start_binding)
         raise TypeError(f"The device is not fakable: {device_id}")
 
-    def add_task(self, fnc, *args, delay=None, period=None, **kwargs) -> None:
+    def add_task(self, fnc, *args, delay=None, period=None, **kwargs) -> asyncio.Task:
         """Start a task after delay seconds and then repeat it every period seconds."""
         self._tasks = [t for t in self._tasks if not t.done()]
-        self._tasks.append(
-            schedule_task(fnc, *args, delay=delay, period=period, **kwargs)
-        )
+        task = schedule_task(fnc, *args, delay=delay, period=period, **kwargs)
+        self._tasks.append(task)
+        return task
