@@ -29,9 +29,11 @@ from .const import (
     SZ_BACKOFF,
     SZ_DAEMON,
     SZ_DHW_IDX,
+    SZ_DISABLE_BACKOFF,
     SZ_DOMAIN_ID,
     SZ_FUNC,
     SZ_PRIORITY,
+    SZ_QOS,
     SZ_RETRIES,
     SZ_TIMEOUT,
     SZ_ZONE_IDX,
@@ -94,41 +96,32 @@ class Qos:
 
     POLL_INTERVAL = 0.002
 
-    # tx (from sent to gwy, to get back from gwy) seems to takes appDEFAULT_KEYSrox. 0.025s
-    DEFAULT_TX_TIMEOUT = td(seconds=0.2)  # 0.20 OK, but too high?
-    DEFAULT_TX_RETRIES = 2
-    RETRY_LIMIT_MAX = 5
+    TX_PRIORITY_DEFAULT = Priority.DEFAULT
 
-    DEFAULT_RX_TIMEOUT = td(seconds=0.50)  # 0.20 seems OK, 0.10 too low sometimes
-    MAX_BACKOFF_FACTOR = 2  # i.e. tx_timeout 2 ** MAX_BACKOFF
+    # tx (from sent to gwy, to get back from gwy) seems to takes approx. 0.025s
+    TX_RETRIES_DEFAULT = 2
+    TX_RETRIES_MAX = 5
+    TX_TIMEOUT_DEFAULT = td(seconds=0.2)  # 0.20 OK, but too high?
+
+    RX_TIMEOUT_DEFAULT = td(seconds=0.50)  # 0.20 seems OK, 0.10 too low sometimes
+
+    TX_BACKOFFS_MAX = 2  # i.e. tx_timeout 2 ** MAX_BACKOFF
 
     QOS_KEYS = (SZ_PRIORITY, SZ_RETRIES, SZ_TIMEOUT, SZ_BACKOFF)
-
-    DEFAULT_QOS = (Priority.DEFAULT, DEFAULT_TX_RETRIES, DEFAULT_TX_TIMEOUT, True)
-    DEFAULT_QOS_TABLE = (
-        {  # priority, retries, timeout, (enable_)backoff, c.f. DEFAULT_QOS
-            f"{RQ}|{Code._0016}": (Priority.HIGH, 5, None, True),
-            f"{RQ}|{Code._0006}": (Priority.HIGH, 5, None, True),
-            f"{I_}|{Code._0404}": (
-                Priority.HIGH,
-                3,
-                td(seconds=0.30),
-                True,
-            ),  # short Tx
-            f"{RQ}|{Code._0404}": (Priority.HIGH, 3, td(seconds=1.00), True),
-            f"{W_}|{Code._0404}": (
-                Priority.HIGH,
-                3,
-                td(seconds=1.00),
-                True,
-            ),  # but long Rx
-            f"{RQ}|{Code._0418}": (Priority.LOW, 3, None, None),
-            f"{RQ}|{Code._1F09}": (Priority.HIGH, 5, None, True),
-            f"{I_}|{Code._1FC9}": (Priority.HIGH, 2, td(seconds=1), False),
-            f"{RQ}|{Code._3220}": (Priority.DEFAULT, 1, td(seconds=1.2), False),
-            f"{W_}|{Code._3220}": (Priority.HIGH, 3, td(seconds=1.2), False),
-        }
-    )  # The long timeout for the OTB is for total RTT to slave (boiler)
+    # priority, retries, rx_timeout, backoff
+    DEFAULT_QOS = (Priority.DEFAULT, TX_RETRIES_DEFAULT, TX_TIMEOUT_DEFAULT, True)
+    DEFAULT_QOS_TABLE = {
+        f"{RQ}|{Code._0016}": (Priority.HIGH, 5, None, True),
+        f"{RQ}|{Code._0006}": (Priority.HIGH, 5, None, True),
+        f"{I_}|{Code._0404}": (Priority.HIGH, 3, td(seconds=0.30), True),
+        f"{RQ}|{Code._0404}": (Priority.HIGH, 3, td(seconds=1.00), True),
+        f"{W_}|{Code._0404}": (Priority.HIGH, 3, td(seconds=1.00), True),
+        f"{RQ}|{Code._0418}": (Priority.LOW, 3, None, None),
+        f"{RQ}|{Code._1F09}": (Priority.HIGH, 5, None, True),
+        f"{I_}|{Code._1FC9}": (Priority.HIGH, 2, td(seconds=1), False),
+        f"{RQ}|{Code._3220}": (Priority.DEFAULT, 1, td(seconds=1.2), False),
+        f"{W_}|{Code._3220}": (Priority.HIGH, 3, td(seconds=1.2), False),
+    }  # The long timeout for the OTB is for total RTT to slave (boiler)
 
     def __init__(
         self,
@@ -139,13 +132,13 @@ class Qos:
         backoff=None,
     ) -> None:
 
-        self.priority = priority if priority is not None else self.DEFAULT_QOS[0]
-        self.retry_limit = retries if retries is not None else self.DEFAULT_QOS[1]
-        self.tx_timeout = self.DEFAULT_TX_TIMEOUT
-        self.rx_timeout = timeout if timeout is not None else self.DEFAULT_QOS[2]
-        self.disable_backoff = not (
-            backoff if backoff is not None else self.DEFAULT_QOS[3]
-        )
+        self.priority = self.DEFAULT_QOS[0] if priority is None else priority
+        self.retry_limit = self.DEFAULT_QOS[1] if retries is None else retries
+        self.tx_timeout = self.TX_TIMEOUT_DEFAULT
+        self.rx_timeout = self.DEFAULT_QOS[2] if timeout is None else timeout
+        self.disable_backoff = not (self.DEFAULT_QOS[3] if backoff is None else backoff)
+
+        self.retry_limit = min(self.retry_limit, Qos.TX_RETRIES_MAX)
 
     @classmethod  # constructor from verb|code pair
     def verb_code(cls, verb, code, **kwargs) -> Qos:
@@ -275,7 +268,7 @@ def _normalise_until(mode, _, until, duration) -> tuple[Any, Any]:
 
 
 def _qos_params(verb: _VerbT, code: _CodeT, qos: dict) -> Qos:
-    """Class constrcutor wrapped to prevent cyclic reference."""
+    """Class constructor wrapped to prevent cyclic reference."""
     return Qos.verb_code(verb, code, **qos)
 
 
@@ -296,13 +289,11 @@ class Command(Frame):
 
         # used by app layer: callback (protocol.py: func, args, daemon, timeout)
         self._cbk = callback or {}
+        # used by msg layer (for which cmd to send next, with _qos.priority)
+        self._dtm = dt_now()
         # used by pkt layer: qos (transport.py: backoff, priority, retries, timeout)
         self._qos = _qos_params(self.verb, self.code, qos or {})
 
-        # used for by msg layer (for which cmd to send next)
-        self._priority = self._qos.priority  # TODO: should only be a QoS attr
-        self._dtm = dt_now()
-
         self._rx_header: None | str = None
         self._source_entity = None
 
@@ -442,16 +433,20 @@ class Command(Frame):
     def __eq__(self, other: Any) -> bool:
         if not self._is_valid_operand(other):
             return NotImplemented
-        return (self._priority, self._dtm) == (other._priority, other._dtm)
+        return (self._qos.priority, self._dtm) == (other._qos.priority, other._dtm)
 
     def __lt__(self, other: Any) -> bool:
         if not self._is_valid_operand(other):
             return NotImplemented
-        return (self._priority, self._dtm) < (other._priority, other._dtm)
+        return (self._qos.priority, self._dtm) < (other._qos.priority, other._dtm)
 
     @staticmethod
     def _is_valid_operand(other: Any) -> bool:
-        return hasattr(other, "_priority") and hasattr(other, "_dtm")
+        return (
+            hasattr(other, "_dtm")
+            and hasattr(other, "_qos")
+            and hasattr(other._qos, "priority")
+        )
 
     @property
     def tx_header(self) -> _HeaderT:
@@ -1222,7 +1217,7 @@ class Command(Frame):
     def put_bind(
         cls,
         verb: _VerbT,
-        codes: _CodeT | Iterable[_CodeT],
+        codes: None | _CodeT | Iterable[_CodeT],
         src_id: _DeviceIdT,
         *,
         idx="00",
@@ -1236,6 +1231,7 @@ class Command(Frame):
         # .I --- 34:021943 01:145038 --:------ 1FC9 006 00-2309-8855B7  # or, simply: 00
 
         hex_id = Address.convert_to_hex(src_id)
+        codes = [] if codes is None else codes  # TODO: untidy
         codes = ([codes] if isinstance(codes, _CodeT) else list(codes)) + [Code._1FC9]
 
         if dst_id is None and verb == I_:
@@ -1253,7 +1249,7 @@ class Command(Frame):
         else:
             raise ValueError("Invalid parameters")
 
-        kwargs["qos"] = {"priority": Priority.HIGH, "retries": 3}
+        kwargs[SZ_QOS] = {SZ_PRIORITY: Priority.HIGH, SZ_RETRIES: 3}
         return cls._from_attrs(
             verb, Code._1FC9, payload, addr0=src_id, addr1=dst_id, addr2=addr2, **kwargs
         )
@@ -1312,7 +1308,14 @@ class Command(Frame):
         """
         # .I --- 34:021943 --:------ 34:021943 30C9 003 000C0D
 
-        if dev_id[:2] != DEV_TYPE_MAP.HCW:
+        if dev_id[:2] not in (
+            DEV_TYPE_MAP.TR0,  # 00
+            DEV_TYPE_MAP.HCW,  # 03
+            DEV_TYPE_MAP.TRV,  # 04
+            DEV_TYPE_MAP.DTS,  # 12
+            DEV_TYPE_MAP.DT2,  # 22
+            DEV_TYPE_MAP.RND,  # 34
+        ):
             raise TypeError(
                 f"Faked device {dev_id} has an unsupported device type: "
                 f"device_id should be like {DEV_TYPE_MAP.HCW}:xxxxxx"
@@ -1374,12 +1377,12 @@ class Command(Frame):
 
         assert msg_type in LOOKUP_PUZZ, f"Invalid/deprecated Puzzle type: {msg_type}"
 
-        qos = kwargs.get("qos", {})
-        qos["priority"] = qos.get("priority", Priority.HIGHEST)
+        qos = kwargs.get(SZ_QOS, {})
+        qos[SZ_PRIORITY] = qos.get(SZ_PRIORITY, Priority.HIGHEST)
         if msg_type == "10":
-            qos["disable_backoff"] = qos.get("disable_backoff", True)
-            qos["retries"] = qos.get("retries", 12)
-        kwargs["qos"] = qos
+            qos[SZ_DISABLE_BACKOFF] = qos.get(SZ_DISABLE_BACKOFF, True)
+            qos[SZ_RETRIES] = qos.get(SZ_RETRIES, 12)
+        kwargs[SZ_QOS] = qos
 
         payload = f"00{msg_type}"
 
