@@ -14,7 +14,9 @@ DEV_MODE = __dev_mode__
 
 # used by tansport QoS...
 SZ_BACKOFF = "backoff"
+SZ_DISABLE_BACKOFF = "disable_backoff"
 SZ_PRIORITY = "priority"
+SZ_QOS = "qos"
 SZ_RETRIES = "retries"
 SZ_TIMEOUT = "timeout"
 
@@ -65,9 +67,18 @@ SZ_PRESENCE_DETECTED = "presence_detected"
 
 
 def slug(string: str) -> str:
+    """Convert a string to snake_case."""
     return re.sub(r"[\W_]+", "_", string.lower())
 
 
+def _alt_slugify_string(key: str) -> str:
+    """Convert a string to snake_case."""
+    string = re.sub(r"[\-\.\s]", "_", str(key))
+    return (string[0]).lower() + re.sub(
+        r"[A-Z]", lambda matched: f"_{matched.group(0).lower()}", string[1:]  # type: ignore[str-bytes-safe]
+    )
+
+
 class AttrDict(dict):
     _SZ_AKA_SLUG = "_root_slug"
     _SZ_DEFAULT = "_default"
@@ -424,6 +435,7 @@ SYS_MODE_MAP = attr_dict_factory(
 
 SZ_ACTUATOR = "actuator"
 SZ_ACTUATORS = "actuators"
+SZ_BINDINGS = "bindings"
 SZ_DATETIME = "datetime"
 SZ_DEVICE_CLASS = "device_class"  # used in 0418 only?
 SZ_DEVICE_ID = "device_id"
@@ -481,14 +493,16 @@ F6, F7, F8, F9, FA, FB, FC, FD, FE, FF = (f"{x:02X}" for x in range(0xF6, 0x100)
 
 DOMAIN_TYPE_MAP = {
     F6: "cooling_valve",  # cooling
-    F8: None,
+    F7: "domain_f7",
+    F8: "domain_f8",
     F9: DEV_ROLE_MAP[DEV_ROLE.HT1],  # Heating Valve
     FA: DEV_ROLE_MAP[DEV_ROLE.HTG],  # HW Valve (or UFH loop if src.type == UFC?)
-    FB: None,
+    FB: "domain_fb",  # also: cooling valve?
     FC: DEV_ROLE_MAP[DEV_ROLE.APP],  # appliance_control
-    FD: "unknown",  # seen with hometronics
+    FD: "domain_fd",  # seen with hometronics
+    # "FE": ???
     # FF: "system",  # TODO: remove this, is not a domain
-}  # "21": "Ventilation",
+}  # "21": "Ventilation", "88": ???
 DOMAIN_TYPE_LOOKUP = {v: k for k, v in DOMAIN_TYPE_MAP.items() if k != FF}
 
 DHW_STATE_MAP = {"00": "off", "01": "on"}
