@@ -22,6 +22,7 @@ from .const import (
     SZ_ACTUATOR,
     SZ_AIR_QUALITY,
     SZ_AIR_QUALITY_BASE,
+    SZ_BINDINGS,
     SZ_BYPASS_POSITION,
     SZ_CHANGE_COUNTER,
     SZ_CO2_LEVEL,
@@ -83,7 +84,7 @@ from .helpers import (
     double_from_hex,
     dtm_from_hex,
     dts_from_hex,
-    flag8,
+    flag8_from_hex,
     percent_from_hex,
     str_from_hex,
     temp_from_hex,
@@ -255,11 +256,13 @@ def parser_0005(payload, msg) -> dict | list[dict]:  # TODO: needs a cleanup
 
     def _parser(seqx) -> dict:
         if msg.src.type == DEV_TYPE_MAP.UFC:  # DEX, or use: seqx[2:4] == ...
-            zone_mask = flag8(seqx[6:8], lsb=True)
+            zone_mask = flag8_from_hex(seqx[6:8], lsb=True)
         elif msg.len == 3:  # ATC928G1000 - 1st gen monochrome model, max 8 zones
-            zone_mask = flag8(seqx[4:6], lsb=True)
+            zone_mask = flag8_from_hex(seqx[4:6], lsb=True)
         else:
-            zone_mask = flag8(seqx[4:6], lsb=True) + flag8(seqx[6:8], lsb=True)
+            zone_mask = flag8_from_hex(seqx[4:6], lsb=True) + flag8_from_hex(
+                seqx[6:8], lsb=True
+            )
         zone_class = ZON_ROLE_MAP.get(seqx[2:4], DEV_ROLE_MAP[seqx[2:4]])
         return {
             SZ_ZONE_TYPE: seqx[2:4],  # TODO: ?remove & keep zone_class?
@@ -586,7 +589,9 @@ def parser_0404(payload, msg) -> dict:
 
     assert payload[4:6] in ("00", payload[:2]), _INFORM_DEV_MSG
 
-    if int(payload[8:10], 16) * 2 != len(payload[14:]):
+    if int(payload[8:10], 16) * 2 != (frag_length := len(payload[14:])) and (
+        msg.verb != I_ or frag_length != 0
+    ):
         raise InvalidPayloadError(f"Incorrect fragment length: 0x{payload[8:10]}")
 
     if msg.verb == RQ:  # have a ctx: idx|frag_idx
@@ -1197,7 +1202,7 @@ def parser_1f41(payload, msg) -> dict:
 
 
 @parser_decorator  # programme_config, HVAC
-def parser_1F70(payload, msg) -> dict:
+def parser_1f70(payload, msg) -> dict:
     # Seen on Orcon: see 1470, 1F70, 22B0
 
     try:
@@ -1285,19 +1290,29 @@ def parser_1fc9(payload, msg) -> list:
             assert int(seqx[:2], 16) < 16
         return [seqx[:2], seqx[2:6], hex_id_to_dev_id(seqx[6:])]
 
+    if msg.verb == I_ and msg.src is msg.dst:
+        bind_step = "offer"
+    elif msg.verb == W_ and msg.src is not msg.dst:
+        bind_step = "accept"
+    elif msg.verb == I_:
+        bind_step = "confirm"  # payload could be "00"
+    else:
+        bind_step = None  # unknown
+
     if payload == "00":
-        return []
+        return {"phase": bind_step, SZ_BINDINGS: []}
 
     assert msg.len >= 6 and msg.len % 6 == 0, msg.len  # assuming not RQ
     assert msg.verb in (I_, W_, RP), msg.verb  # devices will respond to a RQ!
     # assert (
     #     msg.src.id == hex_id_to_dev_id(payload[6:12])
     # ), f"{payload[6:12]} ({hex_id_to_dev_id(payload[6:12])})"  # NOTE: use_regex
-    return [
+    bindings = [
         _parser(payload[i : i + 12])
         for i in range(0, len(payload), 12)
         # if payload[i : i + 2] != "90"  # TODO: WIP, what is 90?
     ]
+    return {"phase": bind_step, SZ_BINDINGS: bindings}
 
 
 @parser_decorator  # unknown_1fca, HVAC?
@@ -1416,11 +1431,17 @@ def parser_22c9(payload, msg) -> list:
 @parser_decorator  # unknown_22d0, HVAC system switch?
 def parser_22d0(payload, msg) -> dict:
 
-    # 2020-03-02T19:20:02.716392 056  I --- 02:001107 --:------ 02:001107 22D0 004 00000002          # an UFC
+    # When closing H/C contact (or enabling cooling mode with buttons when H/C contact is closed) on HCE80 it sends following packet:
+    # .I — 02:044994 --:------ 02:044994 22D0 004 0010000A < AssertionError…
 
-    # 2022-07-28T13:25:40.196523 074  W --- 21:064743 02:250708 --:------ 22D0 008 0314001E-14030020
-    # 2022-07-28T13:25:40.237555 045  I --- 02:250708 21:064743 --:------ 22D0 004 03130000
-    # 2022-07-28T13:25:40.642563 045  I --- 02:250708 --:------ 02:250708 22D0 004 00130000          # sends 3x, 1s apart
+    # When H/C contact is opened, it send the following packet (although this packet is not transmitted, when cooling mode is disabled with buttons):
+    # .I — 02:044994 --:------ 02:044994 22D0 004 0000000A < AssertionError…
+
+    # .I --- 02:001107 --:------ 02:001107 22D0 004 00000002          # an UFC
+
+    # .W --- 21:064743 02:250708 --:------ 22D0 008 0314001E-14030020
+    # .I --- 02:250708 21:064743 --:------ 22D0 004 03130000
+    # .I --- 02:250708 --:------ 02:250708 22D0 004 00130000          # sends 3x, 1s apart
 
     assert payload in (
         "00000002",
@@ -1611,7 +1632,7 @@ def parser_22f3(payload, msg) -> dict:
     if msg.len >= 3:
         result = {
             "minutes" if units != "index" else "index": duration,
-            "flags": flag8(payload[2:4]),
+            "flags": flag8_from_hex(payload[2:4]),
             "_new_speed_mode": new_speed,
             "_fallback_speed_mode": fallback_speed,
         }
@@ -1743,7 +1764,9 @@ def parser_2401(payload, msg) -> dict:
 
     try:
         assert payload[2:4] == "00", f"byte 1: {payload[2:4]}"
-        assert int(payload[4:6], 16) & 0b11110000 == 0, f"byte 2: {flag8(payload[4:6])}"
+        assert (
+            int(payload[4:6], 16) & 0b11110000 == 0
+        ), f"byte 2: {flag8_from_hex(payload[4:6])}"
         assert int(payload[6:], 0x10) <= 200, f"byte 3: {payload[6:]}"
     except AssertionError as exc:
         _LOGGER.warning(f"{msg!r} < {_INFORM_DEV_MSG} ({exc})")
@@ -1751,7 +1774,7 @@ def parser_2401(payload, msg) -> dict:
     return {
         SZ_PAYLOAD: payload,
         "_value_2": int(payload[4:6], 0x10),
-        "_flags_2": flag8(payload[4:6]),
+        "_flags_2": flag8_from_hex(payload[4:6]),
         "_percent_3": percent_from_hex(payload[6:]),
     }
 
@@ -2061,7 +2084,7 @@ def parser_31d9(payload, msg) -> dict:
         "filter_dirty": bool(bitmap & 0x20),
         "frost_cycle": bool(bitmap & 0x40),
         "has_fault": bool(bitmap & 0x80),
-        "_flags": flag8(payload[2:4]),
+        "_flags": flag8_from_hex(payload[2:4]),
     }
 
     if msg.len == 3:  # usu: I -->20: (no seq#)
@@ -2407,27 +2430,31 @@ def parser_3b00(payload, msg) -> dict:
 @parser_decorator  # actuator_state
 def parser_3ef0(payload, msg) -> dict:
 
-    if msg.src.type in DEV_TYPE_MAP.JIM:  # Honeywell Jasper, DEX
+    if msg.src.type == DEV_TYPE_MAP.JIM:  # Honeywell Jasper
         assert msg.len == 20, f"expecting len 20, got: {msg.len}"
         return {
             "ordinal": f"0x{payload[2:8]}",
             "blob": payload[8:],
         }
 
-    assert msg.len in (3, 6, 9)  # [2 + (4, 10, 16)]/2
+    # TODO: These two should be picked up by the regex
+    assert msg.len in (3, 6, 9), f"Invalid payload length: {msg.len}"
+    assert payload[:2] == "00", f"Invalid payload context: {payload[:2]}"
 
-    if msg.len == 3:  # I|BDR|003
+    if msg.len == 3:  # I|BDR|003 (the following are the only two payloads ever seen)
         # .I --- 13:042805 --:------ 13:042805 3EF0 003 0000FF
         # .I --- 13:023770 --:------ 13:023770 3EF0 003 00C8FF
-        assert payload[2:4] in ("00", "C8"), f"byte 1: {payload[2:4]}"
-        assert payload[4:6] == "FF", f"byte 2: {payload[4:6]}"
-        mod_level = percent_from_hex(payload[2:4])
+        assert payload[2:4] in ("00", "C8"), f"byte 1: {payload[2:4]} (not 00/C8)"
+        assert payload[4:6] == "FF", f"byte 2: {payload[4:6]} (not FF)"
+        mod_level = percent_from_hex(payload[2:4])  # , high_res=True)
 
-    if msg.len >= 6:  # RP|OTB|006 (to RQ|CTL/HGI/RFG)
-        # RP --- 10:105624 01:133689 --:------ 3EF0 006 0000100000FF
-        # RP --- 10:105624 01:133689 --:------ 3EF0 006 003B100C00FF
-        assert payload[4:6] in ("00", "10", "11"), f"byte 2: {payload[4:6]}"
-        mod_level = percent_from_hex(payload[2:4], high_res=False)
+    else:  # msg.len >= 6:  # RP|OTB|006 (to RQ|CTL/HGI/RFG)
+        # RP --- 10:004598 34:003611 --:------ 3EF0 006 0000100000FF
+        # RP --- 10:004598 34:003611 --:------ 3EF0 006 0000110000FF
+        # RP --- 10:138822 01:187666 --:------ 3EF0 006 0064100C00FF
+        # RP --- 10:138822 01:187666 --:------ 3EF0 006 0064100200FF
+        assert payload[4:6] in ("10", "11"), f"byte 2: {payload[4:6]}"  # maybe 00 too?
+        mod_level = percent_from_hex(payload[2:4], high_res=False)  # 00-64 (or FF)
 
     result = {
         "modulation_level": mod_level,  # 0008[2:4], 3EF1[10:12]
@@ -2438,20 +2465,15 @@ def parser_3ef0(payload, msg) -> dict:
         # RP --- 10:138822 01:187666 --:------ 3EF0 006 000110FA00FF  # ?corrupt
 
         # for OTB (there's no reliable) modulation_level <-> flame_state)
-        # assert (
-        #     payload[6:8] == "FF" or int(payload[6:8], 16) & 0b11110000 == 0
-        # ), f"byte 3: {payload[6:8]}"
-        assert int(payload[8:10], 16) & 0b11110000 == 0, f"byte 4: {payload[8:10]}"
-        # assert payload[10:12] in ("00", "1C", "FF"), f"byte 5: {payload[10:12]}"
 
         result.update(
             {
-                "_flags_3": flag8(payload[6:8]),
+                "_flags_3": flag8_from_hex(payload[6:8]),
                 "ch_active": bool(int(payload[6:8], 0x10) & 1 << 1),
                 "dhw_active": bool(int(payload[6:8], 0x10) & 1 << 2),
                 "flame_active": bool(int(payload[6:8], 0x10) & 1 << 3),  # flame_on
-                f"_{SZ_UNKNOWN}_4": payload[8:10],
-                f"_{SZ_UNKNOWN}_5": payload[10:12],  # rel_modulation?
+                "_unknown_4": payload[8:10],  # FF, 00, 01, 0A
+                "_unknown_5": payload[10:12],  # FF, 1C, ?others
             }
         )
 
@@ -2463,20 +2485,38 @@ def parser_3ef0(payload, msg) -> dict:
 
         result.update(
             {
-                "_flags_6": flag8(payload[12:14]),
+                "_flags_6": flag8_from_hex(payload[12:14]),
                 "ch_enabled": bool(int(payload[12:14], 0x10) & 1 << 0),
                 "ch_setpoint": int(payload[14:16], 0x10),
                 "max_rel_modulation": percent_from_hex(payload[16:18], high_res=False),
             }
         )
 
-    try:
+    try:  # Trying to decode flags...
+        # assert payload[4:6] != "11" or (
+        #     payload[2:4] == "00"
+        # ), f"bytes 1+2: {payload[2:6]}"  # 97% is 00 when 11, but not always
+
+        assert payload[4:6] in ("FF", "10", "11"), f"byte 2: {payload[4:6]}"
+
         assert "_flags_3" not in result or (
-            [result["_flags_3"][i] for i in (0, 1, 2, 3)] == [0] * 4
-        ), result["_flags_3"]
+            payload[6:8] == "FF" or int(payload[6:8], 0x10) & 0b10110000 == 0
+        ), f'byte 3: {result["_flags_3"]}'
+        # only 01:10:040239 does 0b01000000
+
+        assert "_unknown_4" not in result or (
+            payload[8:10] in ("FF", "00", "01", "04", "0A")
+        ), f"byte 4: {payload[8:10]}"
+        # only 10:040239 does 04
+
+        assert "_unknown_5" not in result or (
+            payload[10:12] in ("00", "1C", "FF")
+        ), f"byte 5: {payload[10:12]}"
+
         assert "_flags_6" not in result or (
-            [result["_flags_6"][i] for i in (0, 1, 2, 3, 4, 5)] == [0] * 6
-        ), result["_flags_6"]
+            int(payload[12:14], 0x10) & 0b11111100 == 0
+        ), f'byte 6: {result["_flags_6"]}'
+
     except AssertionError as exc:
         _LOGGER.warning(
             f"{msg!r} < {_INFORM_DEV_MSG} ({exc}), with a description of your system"
@@ -2629,5 +2669,5 @@ def parser_unknown(payload, msg) -> dict:
 PAYLOAD_PARSERS = {
     k[7:].upper(): v
     for k, v in locals().items()
-    if callable(v) and k.startswith("parser_") and k != "parser_unknown"
+    if callable(v) and k.startswith("parser_") and len(k) == 11
 }
