@@ -21,6 +21,10 @@ from .exceptions import CorruptStateError, InvalidPacketError
 from .message import Message
 from .packet import Packet
 
+_MessageProtocolT = TypeVar("_MessageProtocolT", bound="MessageProtocol")
+_MessageTransportT = TypeVar("_MessageTransportT", bound="MessageTransport")
+
+
 DONT_CREATE_MESSAGES = 3  # duplicate
 
 SZ_WRITER_TASK = "writer_task"
@@ -33,10 +37,6 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
-_MessageProtocolT = TypeVar("_MessageProtocolT", bound="MessageProtocol")
-_MessageTransportT = TypeVar("_MessageTransportT", bound="MessageTransport")
-
-
 class CallbackAsAwaitable:
     """Create an pair of functions so that the callback can be awaited.
 
@@ -46,8 +46,8 @@ class CallbackAsAwaitable:
     The callback (putter) may put the message in the queue before the getter is invoked.
     """
 
-    SAFETY_TIMEOUT_DEFAULT = 30.0  # used to prevent waiting forever
-    SAFETY_TIMEOUT_MINIMUM = 10.0
+    SAFETY_TIMEOUT_DEFAULT = 9.9  # used to prevent waiting forever
+    SAFETY_TIMEOUT_MINIMUM = 5.0
     HAS_TIMED_OUT = False
     SHORT_WAIT = 0.001  # seconds
 
@@ -61,8 +61,8 @@ class CallbackAsAwaitable:
     async def getter(self, timeout: float = SAFETY_TIMEOUT_DEFAULT) -> Message:
         """Poll the queue until the message arrives, or the timer expires."""
 
-        if timeout is None or timeout <= self.SAFETY_TIMEOUT_MINIMUM:
-            timeout = self.SAFETY_TIMEOUT_DEFAULT
+        if timeout <= self.SAFETY_TIMEOUT_MINIMUM:
+            timeout = self.SAFETY_TIMEOUT_MINIMUM
         self.expires = dt.now() + td(seconds=timeout)
 
         while dt.now() < self.expires:
@@ -81,15 +81,10 @@ class CallbackAsAwaitable:
         return msg
 
     # the callback...
-    def putter(self, msg: Message, timeout: float = SAFETY_TIMEOUT_DEFAULT) -> None:
+    def putter(self, msg: Message) -> None:
         """Put the message in the queue (when invoked)."""
 
-        if timeout is None or timeout <= self.SAFETY_TIMEOUT_MINIMUM:
-            timeout = self.SAFETY_TIMEOUT_DEFAULT
-        self.expires = dt.now() + td(seconds=timeout)
-
-        # self._queue.put_nowait(msg)  # ...so should not raise Full
-        self._loop.call_soon_threadsafe(self._queue.put_nowait, msg)
+        self._queue.put_nowait(msg)
 
 
 def awaitable_callback(loop) -> tuple[Callable[..., Awaitable[Message]], Callable]:
@@ -156,7 +151,7 @@ class MessageTransport(asyncio.Transport):
         async def call_send_data(cmd):
             _LOGGER.debug("MsgTransport.pkt_dispatcher(%s): send_data", cmd)
             if cmd._cbk:
-                self._add_callback(cmd.rx_header, cmd._cbk)
+                self._add_callback(cmd.rx_header or cmd.tx_header, cmd._cbk, cmd=cmd)
 
             if _LOGGER.getEffectiveLevel() == logging.INFO:  # i.e. don't log for DEBUG
                 _LOGGER.info("SENT: %s", cmd)
@@ -196,12 +191,18 @@ class MessageTransport(asyncio.Transport):
 
         return self._extra[self.WRITER]
 
-    def _add_callback(self, header: str, callback: dict) -> None:
+    def _add_callback(
+        self, header: str, callback: dict, cmd: None | Command = None
+    ) -> None:
+        # assert header not in self._callbacks  # CBK, below
+        # self._callbacks[header] = CallbackWrapper(header, callback, cmd=cmd)
+
         callback[SZ_EXPIRES] = (
             dt.max
             if callback.get(SZ_DAEMON)
             else dt.now() + td(seconds=callback.get(SZ_TIMEOUT, 1))
         )
+        callback["cmd"] = cmd
         self._callbacks[header] = callback
 
     def _pkt_receiver(self, pkt: Packet) -> None:
@@ -211,12 +212,9 @@ class MessageTransport(asyncio.Transport):
             _LOGGER.info("rcvd: %s", pkt)
 
         # HACK: 1st, notify all expired callbacks
-        for (
-            hdr,
-            callback,
-        ) in self._callbacks.items():
-            if callback.get(SZ_EXPIRES, dt.max) < pkt.dtm and not callback.get(
-                SZ_EXPIRED
+        for hdr, callback in self._callbacks.items():
+            if not callback.get(SZ_EXPIRED) and (
+                callback.get(SZ_EXPIRES, dt.max) < pkt.dtm
             ):
                 # see also: PktProtocolQos.send_data()
                 (_LOGGER.warning if DEV_MODE else _LOGGER.info)(
@@ -246,8 +244,7 @@ class MessageTransport(asyncio.Transport):
 
         # HACK: 3rd, invoke any callback
         # NOTE: msg._pkt._hdr is expensive - don't call it unless there's callbacks
-        if self._callbacks and msg._pkt._hdr in self._callbacks:
-            callback = self._callbacks[msg._pkt._hdr]
+        if self._callbacks and (callback := self._callbacks.get(msg._pkt._hdr)):  # type: ignore[assignment]
             callback[SZ_FUNC](msg)  # ZX: 2/3
             if not callback.get(SZ_DAEMON):
                 del self._callbacks[msg._pkt._hdr]
@@ -591,7 +588,11 @@ def create_protocol_factory(
 
 
 def create_msg_stack(
-    gwy, msg_callback: Callable, protocol_factory: Callable = None
+    gwy,
+    msg_callback: Callable[[Message, Optional[Message]], None],
+    /,
+    *,
+    protocol_factory: Callable[[], _MessageProtocolT] = None,
 ) -> tuple[_MessageProtocolT, _MessageTransportT]:
     """Utility function to provide a transport to a client protocol.
 
