@@ -34,7 +34,7 @@ from queue import Queue
 from string import printable  # ascii_letters, digits
 from time import perf_counter
 from types import SimpleNamespace
-from typing import Awaitable, Callable, Iterable, Optional, TextIO, TypeVar
+from typing import Any, Awaitable, Callable, Iterable, TextIO, TypeVar
 
 from serial import SerialBase, SerialException, serial_for_url  # type: ignore[import]
 from serial_asyncio import SerialTransport as SerTransportAsync  # type: ignore[import]
@@ -44,7 +44,7 @@ from .command import Command, Qos
 
 # skipcq: PY-W2000
 from .const import DEV_TYPE, DEV_TYPE_MAP, SZ_DEVICE_ID, __dev_mode__
-from .exceptions import InvalidPacketError
+from .exceptions import ForeignGatewayError, InvalidPacketError
 from .helpers import dt_now
 from .packet import Packet
 from .protocol import create_protocol_factory
@@ -73,6 +73,9 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     Code,
 )
 
+_PacketProtocolT = TypeVar("_PacketProtocolT", bound="PacketProtocolBase")
+_PacketTransportT = TypeVar("_PacketTransportT", bound=asyncio.BaseTransport)
+
 
 DEV_MODE = __dev_mode__ and False  # debug is_wanted, or qos_fx
 DEV_HACK_REGEX = False
@@ -123,10 +126,6 @@ VALID_CHARACTERS = printable  # "".join((ascii_letters, digits, ":-<*# "))
 # !FS - save autotune
 
 
-_PacketProtocolT = TypeVar("_PacketProtocolT", bound=asyncio.BaseProtocol)
-_PacketTransportT = TypeVar("_PacketTransportT", bound=asyncio.BaseTransport)
-
-
 def _str(value: bytes) -> str:
     try:
         result = "".join(
@@ -206,7 +205,7 @@ def avoid_system_syncs(fnc: Callable[..., Awaitable]):
                 < SYNC_WINDOW_UPPER
             )
 
-        start = perf_counter()
+        start = perf_counter()  # TODO: remove
 
         # wait for the start of the sync cycle (I|1F09|003, Tx time ~0.009)
         while any(is_imminent(p) for p in sync_cycles):
@@ -215,7 +214,7 @@ def avoid_system_syncs(fnc: Callable[..., Awaitable]):
         # wait for the remainder of sync cycle (I|2309/30C9) to complete
         if (x := perf_counter() - start) > SYNC_WAIT_SHORT:
             await asyncio.sleep(SYNC_WAIT_LONG)
-            # FIXME: remove this block
+            # FIXME: remove this block, and merge both ifs
             times_0.append(x)
             _LOGGER.warning(
                 f"*** sync cycle stats: {x:.3f}, "
@@ -231,7 +230,7 @@ def avoid_system_syncs(fnc: Callable[..., Awaitable]):
 
 
 def track_system_syncs(fnc: Callable):
-    """Track/remember the most recent sync cycle for a controller."""
+    """Track/remember the any new/outstanding TCS sync cycle."""
 
     MAX_SYNCS_TRACKED = 3
 
@@ -248,9 +247,9 @@ def track_system_syncs(fnc: Callable):
         sync_cycles = deque(
             p for p in sync_cycles if p.src != pkt.src and is_pending(p)
         )
-        sync_cycles.append(pkt)
+        sync_cycles.append(pkt)  # TODO: sort
 
-        if len(sync_cycles) > MAX_SYNCS_TRACKED:
+        if len(sync_cycles) > MAX_SYNCS_TRACKED:  # safety net for corrupted payloads
             sync_cycles.popleft()
 
         fnc(self, pkt, *args, **kwargs)
@@ -350,21 +349,39 @@ class SerTransportBase(asyncio.ReadTransport):
 
     _extra: dict
 
-    def __init__(self, loop, extra=None):
+    def __init__(self, loop: asyncio.AbstractEventLoop, extra=None):
         super().__init__(extra=extra)
 
         self._loop = loop
-        self._extra[SZ_POLLER_TASK] = self._loop.create_task(self._polling_loop())
+
+        task = self._loop.create_task(self._polling_loop())
+        task.add_done_callback(self._handle_polling_loop_result)
+        self._extra[SZ_POLLER_TASK] = task
 
         # for sig in (signal.SIGINT, signal.SIGTERM):
         #     self._loop.add_signal_handler(sig, self.abort)
 
         self._is_closing = False
 
-    async def _polling_loop(self):  # TODO: make into a thread, as doing I/O
+    async def _polling_loop(self) -> None:  # TODO: make into a thread, as doing I/O
+        """Poll the packet source for packets, also calls connection_made/lost()."""
         # self._protocol.connection_made(self)
-
         raise NotImplementedError
+        # self._protocol.connection_lost(None)
+
+    def _handle_polling_loop_result(self, task: asyncio.Task) -> None:
+        """Call connection_lost(exc) if there was an Exception from the polling_loop.
+
+        The polling_loop should call connection_lost(None) if there is no exception.
+        """
+        try:
+            task.result()
+        # except (KeyboardInterrupt, SystemExit):
+        #     pass  # no value in raising
+        except asyncio.CancelledError:
+            pass  # self._protocol.connection_lost(None)
+        except Exception as exc:  # pylint: disable=broad-except
+            self._protocol.connection_lost(exc)
 
     def close(self):
         """Close the transport."""
@@ -373,11 +390,11 @@ class SerTransportBase(asyncio.ReadTransport):
             return
         self._is_closing = True
 
-        self._protocol.pause_writing()
         if task := self._extra.get(SZ_POLLER_TASK):
             task.cancel()
 
         self._loop.call_soon(self._protocol.connection_lost, None)
+        # cause: self._protocol.pause_writing()
 
     def is_closing(self) -> bool:
         """Return True if the transport is closing or closed."""
@@ -387,7 +404,13 @@ class SerTransportBase(asyncio.ReadTransport):
 class SerTransportRead(SerTransportBase):
     """Interface for a packet transport via a dict (saved state) or a file (pkt log)."""
 
-    def __init__(self, loop, protocol: PacketProtocolBase, packet_source, extra=None):
+    def __init__(
+        self,
+        loop: asyncio.AbstractEventLoop,
+        protocol: _PacketProtocolT,
+        packet_source,
+        extra=None,
+    ):
         super().__init__(loop, extra=extra)
 
         self._protocol = protocol
@@ -395,7 +418,8 @@ class SerTransportRead(SerTransportBase):
 
         self._protocol.pause_writing()
 
-    async def _polling_loop(self):  # TODO: harden with try
+    async def _polling_loop(self):  # TODO: harden: wrap d_r() with try
+        """Poll the packet source for packets, also calls connection_made/lost()."""
         self._protocol.connection_made(self)
 
         if isinstance(self._packets, dict):  # can assume dtm_str is OK
@@ -417,21 +441,28 @@ class SerTransportRead(SerTransportBase):
         raise NotImplementedError
 
 
-class SerTransportPoll(SerTransportBase):
+class SerTransportPoll(SerTransportBase, asyncio.WriteTransport):
     """Interface for a packet transport using polling."""
 
     MAX_BUFFER_SIZE = 500
 
-    def __init__(self, loop, protocol: PacketProtocolBase, ser_instance, extra=None):
+    def __init__(
+        self,
+        loop: asyncio.AbstractEventLoop,
+        protocol: _PacketProtocolT,
+        ser_instance: SerialBase,
+        extra=None,
+    ):
         super().__init__(loop, extra=extra)
 
         self._protocol = protocol
         self.serial = ser_instance
 
-        self._is_closing = None
+        self._is_closing: bool = None  # type: ignore[assignment]
         self._write_queue: Queue = Queue(maxsize=self.MAX_BUFFER_SIZE)
 
-    async def _polling_loop(self):
+    async def _polling_loop(self) -> None:
+        """Poll the packet source for packets, also calls connection_made/lost()."""
         self._protocol.connection_made(self)
 
         while self.serial.is_open:
@@ -492,7 +523,7 @@ class PacketProtocolBase(asyncio.Protocol):
         self._include = list(gwy._include.keys()) + [NON_DEV_ADDR.id, NUL_DEV_ADDR.id]
         self._unwanted: list = []  # not: [NON_DEV_ADDR.id, NUL_DEV_ADDR.id]
 
-        self._hgi80 = {
+        self._hgi80: dict[str, Any] = {
             SZ_DEVICE_ID: None,
             SZ_FINGERPRINT: None,
             SZ_IS_EVOFW3: None,
@@ -503,6 +534,8 @@ class PacketProtocolBase(asyncio.Protocol):
             k for k, v in gwy._include.items() if v.get(SZ_CLASS) == DEV_TYPE.HGI
         ]:
             self._hgi80[SZ_KNOWN_HGI] = known_hgis[0]
+        else:
+            _LOGGER.info(f"The {SZ_KNOWN_LIST} should specify the gateway (HGI) device")
 
         self._use_regex = getattr(self._gwy.config, SZ_USE_REGEX, {})
 
@@ -538,14 +571,15 @@ class PacketProtocolBase(asyncio.Protocol):
                 "is strongly recommended)"
             )
 
-    def connection_lost(self, exc: Optional[Exception]) -> None:
+    def connection_lost(self, exc: None | Exception) -> None:
         """Called when the connection is lost or closed."""
-        _LOGGER.debug(f"{self}.connection_lost(exc)")
         # serial.serialutil.SerialException: device reports error (poll)
+        if exc:
+            _LOGGER.error("Exception raised by transport: %s", exc)
 
         self.pause_writing()
 
-        if exc is not None:
+        if exc:
             raise exc
 
     def pause_writing(self) -> None:
@@ -574,39 +608,6 @@ class PacketProtocolBase(asyncio.Protocol):
         for dtm, raw_line in bytes_received(data):
             self._line_received(dtm, _normalise(_str(raw_line)), raw_line)
 
-    def _check_set_hgi80(self, pkt: Packet):
-        """Check/set (a pkt with) src.type == 18: - is it the HGI, or a foreign HGI?"""
-
-        if pkt.src.id == self._hgi80[SZ_DEVICE_ID]:
-            return
-
-        if self._hgi80[SZ_DEVICE_ID]:
-            if pkt.verb != RQ:  # HACK: to reduce logspam
-                return
-
-            _LOGGER.warning(  # more than one GWY
-                f"{pkt} < Appears to be a Foreign gateway: {pkt.src.id} (Active gateway: "
-                f"{self._hgi80[SZ_DEVICE_ID]}): this is unsupported{TIP}"
-            )
-
-            if pkt.src.id not in self._include and self.enforce_include:
-                _LOGGER.info(f"Blacklisting {pkt.src.id} (is Foreign gateway?){TIP}")
-                self._unwanted.append(pkt.src.id)
-            return
-
-        if pkt.payload == self._hgi80[SZ_FINGERPRINT]:
-            self._hgi80[SZ_DEVICE_ID] = pkt.src.id
-            _LOGGER.warning(f"{pkt} < Active gateway set to: {pkt.src.id}")
-
-            if pkt.src.id not in self._include:
-                _LOGGER.info(f"Whitelisting {pkt.src.id} (is Active gateway){TIP}")
-                self._include.append(pkt.src.id)  # NOTE: only time _include is modified
-            return
-
-        if pkt.src.id == self._hgi80[SZ_KNOWN_HGI]:
-            self._hgi80[SZ_DEVICE_ID] = pkt.src.id
-            _LOGGER.warning(f"{pkt} < Active gateway set to: {pkt.src.id}")
-
     def _line_received(self, dtm: dt, line: str, raw_line: bytes) -> None:
 
         if _LOGGER.getEffectiveLevel() == logging.INFO:  # i.e. don't log for DEBUG
@@ -639,44 +640,45 @@ class PacketProtocolBase(asyncio.Protocol):
         Called by data_received(bytes) -> line_received(frame) -> pkt_received(pkt).
         """
 
+        def is_wanted_addrs(src_id: str, dst_id: str) -> bool:
+            """Return True if the packet is not to be filtered out.
+
+            In any one packet, an excluded device_id 'trumps' an included device_id.
+            """
+
+            for dev_id in dict.fromkeys((src_id, dst_id)):  # removes duplicates
+                # _unwanted exists because (in future) stale entries need to be removed
+                if dev_id in self._exclude or dev_id in self._unwanted:
+                    return False
+                if dev_id == self._hgi80[SZ_DEVICE_ID]:  # even if not in include list
+                    continue
+                if dev_id not in self._include and self.enforce_include:
+                    return False
+                if dev_id[:2] != DEV_TYPE_MAP.HGI:
+                    continue
+                if dev_id not in self._include and self._hgi80[SZ_DEVICE_ID]:
+                    self._unwanted.append(dev_id)
+                    raise ForeignGatewayError(
+                        f"Blacklisting a Foreign gateway (or is it HVAC?): {dev_id}"
+                        f" (Active gateway: {self._hgi80[SZ_DEVICE_ID]}){TIP}"
+                    )
+                if dev_id == self._hgi80[SZ_KNOWN_HGI] or (
+                    dev_id == pkt.src.id and pkt.payload == self._hgi80[SZ_FINGERPRINT]
+                ):
+                    self._hgi80[SZ_DEVICE_ID] = dev_id
+                    if dev_id not in self._include:
+                        _LOGGER.warning(f"Active gateway set to: {dev_id}{TIP}")
+            return True
+
         self._this_pkt, self._prev_pkt = pkt, self._this_pkt
 
-        if self._callback and self._is_wanted(pkt.src.id, pkt.dst.id):
-
-            if pkt.src.type == DEV_TYPE_MAP.HGI:
-                self._check_set_hgi80(pkt)
-
-            try:  # why not call soon?
+        try:  # TODO: why not call soon?
+            if self._callback and is_wanted_addrs(pkt.src.id, pkt.dst.id):
                 self._callback(pkt)  # only wanted PKTs to the MSG transport's handler
-
-            except AssertionError as exc:  # protect from upper-layer callbacks
-                _LOGGER.exception("%s < exception from msg layer: %s", pkt, exc)
-
-    def _is_wanted(self, src_id: str, dst_id: str) -> bool:
-        """Parse the packet, return True if the packet is not to be filtered out.
-
-        An unwanted device_id will 'trump' a whitelisted device_id in the same packet
-        because there is a significant chance that the packet is simply corrupt.
-        """
-
-        for dev_id in dict.fromkeys((src_id, dst_id)):
-            if dev_id in self._unwanted:  # TODO: remove entries older than (say) 1w/1d
-                return False
-
-            if dev_id in self._exclude:
-                self._unwanted.append(dev_id)
-                return False
-
-            if dev_id in self._include:
-                continue  # TODO: or break if not self.enforce_include?
-
-            if not self._hgi80[SZ_KNOWN_HGI] and dev_id[:2] == DEV_TYPE_MAP.HGI:
-                continue
-
-            if self.enforce_include:  # TODO: omit if using break?
-                return False
-
-        return True
+        except AssertionError as exc:  # protect from upper-layer callbacks
+            _LOGGER.exception("%s < exception from msg layer: %s", pkt, exc)
+        except ForeignGatewayError as exc:
+            _LOGGER.error("%s < exception from pkt layer: %s", pkt, exc)
 
     async def send_data(self, cmd: Command) -> None:
         raise NotImplementedError
@@ -743,7 +745,7 @@ class PacketProtocolPort(PacketProtocolBase):
             except ValueError:
                 pass
 
-    def connection_lost(self, exc: Optional[Exception]) -> None:
+    def connection_lost(self, exc: None | Exception) -> None:
         """Called when the connection is lost or closed."""
 
         if self._leaker:
@@ -859,7 +861,7 @@ class PacketProtocolQos(PacketProtocolPort):
 
         super()._pkt_received(pkt)
 
-    async def send_data(self, cmd: Command) -> Optional[Packet]:  # type: ignore[override]
+    async def send_data(self, cmd: Command) -> None | Packet:  # type: ignore[override]
         """Called when packets are to be sent (not a callback)."""
 
         if self._disable_sending:
@@ -868,30 +870,30 @@ class PacketProtocolQos(PacketProtocolPort):
         if cmd.src.id != HGI_DEV_ADDR.id:
             await self._alert_is_impersonating(cmd)
 
-        def expires(timeout, disable_backoff, retry_count):
+        def get_expiry_time(timeout, disable_backoff, retry_count):
             """Return a dtm for expiring the Tx (or Rx), with an optional backoff."""
             if disable_backoff:
                 return dt.now() + timeout
-            return dt.now() + timeout * 2 ** min(retry_count, Qos.MAX_BACKOFF_FACTOR)
+            return dt.now() + timeout * 2 ** min(retry_count, Qos.TX_BACKOFFS_MAX)
 
         # self._qos_lock.acquire()
         if self._qos_cmd:
-            raise RuntimeError
+            raise RuntimeError  # TODO: remove me
         self._qos_cmd = cmd
         # self._qos_lock.release()
         self._tx_rcvd = None
 
         retry_count = 0
-        while retry_count <= min(cmd._qos.retry_limit, Qos.RETRY_LIMIT_MAX):  # 5
+        while retry_count <= min(cmd._qos.retry_limit, Qos.TX_RETRIES_MAX):
 
             self._rx_rcvd = None
             await super()._send_data(str(cmd))
 
-            tx_expires = expires(
+            tx_expires = get_expiry_time(
                 cmd._qos.tx_timeout, cmd._qos.disable_backoff, retry_count
             )
             while tx_expires > dt.now():  # Step 1: wait for Tx to echo
-                await asyncio.sleep(Qos.POLL_INTERVAL)
+                await asyncio.sleep(Qos.POLL_INTERVAL)  # 0.002
                 if self._tx_rcvd or self._rx_rcvd:
                     break
             else:
@@ -903,7 +905,7 @@ class PacketProtocolQos(PacketProtocolPort):
 
             rx_expires = dt.now() + cmd._qos.rx_timeout
             while rx_expires > dt.now():  # Step 2: wait for Rx to arrive
-                await asyncio.sleep(0.001)
+                await asyncio.sleep(Qos.POLL_INTERVAL)
                 if self._rx_rcvd:
                     break
             else:
@@ -926,10 +928,10 @@ class PacketProtocolQos(PacketProtocolPort):
 
 def create_pkt_stack(
     gwy,
-    pkt_callback: Callable,
+    pkt_callback: Callable[[Packet], None],
     /,
     *,
-    protocol_factory: Callable = None,
+    protocol_factory: Callable[[], _PacketProtocolT] = None,
     port_name: str = None,
     port_config: dict = None,
     packet_log: TextIO = None,
@@ -980,7 +982,7 @@ def create_pkt_stack(
             "(e.g. linux with a local serial port)"
         )
 
-    def protocol_factory_() -> type[_PacketProtocolT]:
+    def protocol_factory_() -> _PacketProtocolT:
         if packet_log or packet_dict is not None:
             return create_protocol_factory(PacketProtocolFile, gwy, pkt_callback)()
         elif gwy.config.disable_sending:  # NOTE: assumes we wont change our mind
@@ -998,6 +1000,8 @@ def create_pkt_stack(
     if (pkt_source := packet_log or packet_dict) is not None:  # {} is a processable log
         return pkt_protocol, SerTransportRead(gwy._loop, pkt_protocol, pkt_source)
 
+    assert port_name is not None  # instead of: _type: ignore[arg-type]
+    assert port_config is not None  # instead of: _type: ignore[arg-type]
     ser_instance = get_serial_instance(port_name, port_config)
 
     if os.name == "nt" or ser_instance.portstr[:7] in ("rfc2217", "socket:"):
