@@ -10,7 +10,7 @@ from __future__ import annotations
 import logging
 import re
 from types import SimpleNamespace
-from typing import Any, TextIO
+from typing import Any, Callable, TextIO
 
 import voluptuous as vol  # type: ignore[import]
 
@@ -101,8 +101,8 @@ SCH_UFH_IDX = vol.Match(r"^0[0-8]$")
 SCH_ZON_IDX = vol.Match(r"^0[0-9AB]$")  # TODO: what if > 12 zones? (e.g. hometronics)
 
 
-def ErrorRenamedKey(new_key):
-    def renamed_key(node_value):
+def ErrorRenamedKey(new_key: str) -> Callable[[Any], None]:
+    def renamed_key(node_value: Any) -> None:
         raise vol.Invalid(f"the key name has changed: rename it to '{new_key}'")
 
     return renamed_key
@@ -256,7 +256,9 @@ SCH_GATEWAY_DICT = SCH_ENGINE_DICT | {
         int, vol.Range(min=0, max=DONT_CREATE_MESSAGES)
     ),
     vol.Optional(SZ_USE_ALIASES, default=False): bool,
-    vol.Optional(SZ_USE_NATIVE_OT, default=False): bool,
+    vol.Optional(SZ_USE_NATIVE_OT, default="prefer"): vol.Any(
+        "always", "prefer", "avoid", "never"
+    ),
 }
 
 
@@ -424,7 +426,7 @@ def load_schema(gwy, **kwargs) -> None:
     [  # NOTE: class favoured, domain ignored
         _get_device(gwy, device_id)  # domain=key[-4:])
         for key in (SZ_ORPHANS_HEAT, SZ_ORPHANS_HVAC)
-        for device_id in kwargs.pop(key, [])
+        for device_id in kwargs.get(key, [])
     ]  # TODO: pass domain (Heat/HVAC), or generalise to SZ_ORPHANS
 
 
@@ -446,7 +448,7 @@ def load_tcs(gwy, ctl_id: str, schema: dict) -> Any:  # System
     ctl.tcs._update_schema(**schema)  # TODO
 
     for dev_id in schema.get(SZ_UFH_SYSTEM, {}).keys():  # UFH controllers
-        _get_device(gwy, dev_id, parent=ctl)  # , **_schema)
+        _get_device(gwy, dev_id, parent=ctl.tcs)  # , **_schema)
 
     for dev_id in schema.get(SZ_ORPHANS, []):
         _get_device(gwy, dev_id, parent=ctl)
