@@ -54,6 +54,7 @@ from ..protocol import (
     Priority,
 )
 from ..protocol.command import FaultLog, _mk_cmd
+from ..protocol.const import SZ_PRIORITY, SZ_RETRIES
 from ..schemas import (
     DEFAULT_MAX_ZONES,
     SCH_TCS,
@@ -161,19 +162,19 @@ class SystemBase(Parent, Entity):  # 3B00 (multi-relay)
     def __repr__(self) -> str:
         return f"{self.ctl.id} ({self._SLUG})"
 
-    def _setup_discovery_tasks(self) -> None:
-        # super()._setup_discovery_tasks()
+    def _setup_discovery_cmds(self) -> None:
+        # super()._setup_discovery_cmds()
 
         for payload in (
             f"00{DEV_ROLE_MAP.APP}",  # appliance_control
             f"00{DEV_ROLE_MAP.HTG}",  # hotwater_valve
             f"01{DEV_ROLE_MAP.HTG}",  # heating_valve
         ):
-            self._add_discovery_task(
+            self._add_discovery_cmd(
                 _mk_cmd(RQ, Code._000C, payload, self.ctl.id), 60 * 60 * 24, delay=0
             )
 
-        self._add_discovery_task(Command.get_tpi_params(self.id), 60 * 60 * 6, delay=5)
+        self._add_discovery_cmd(Command.get_tpi_params(self.id), 60 * 60 * 6, delay=5)
 
     def _handle_msg(self, msg: Message) -> None:
         def eavesdrop_appliance_control(this, *, prev=None) -> None:
@@ -253,7 +254,7 @@ class SystemBase(Parent, Entity):  # 3B00 (multi-relay)
                     device = None
 
                 if False and device is not None:  # TODO: FIXME
-                    qos = {"priority": Priority.LOW, "retries": 2}
+                    qos = {SZ_PRIORITY: Priority.LOW, SZ_RETRIES: 2}
                     for code in (Code._0008, Code._3EF1):
                         device._make_cmd(code, qos)
 
@@ -369,11 +370,11 @@ class MultiZone(SystemBase):  # 0005 (+/- 000C?)
 
         self._prev_30c9 = None  # used to eavesdrop zone sensors
 
-    def _setup_discovery_tasks(self) -> None:
-        super()._setup_discovery_tasks()
+    def _setup_discovery_cmds(self) -> None:
+        super()._setup_discovery_cmds()
 
         for zone_type in list(ZON_ROLE_MAP.HEAT_ZONES) + [ZON_ROLE_MAP.SEN]:
-            self._add_discovery_task(
+            self._add_discovery_cmd(
                 _mk_cmd(RQ, Code._0005, f"00{zone_type}", self.id),
                 60 * 60 * 24,
                 delay=0,
@@ -394,146 +395,6 @@ class MultiZone(SystemBase):  # 0005 (+/- 000C?)
             ]
 
         def eavesdrop_zone_sensors(this, *, prev=None) -> None:
-            """Determine each zone's sensor by matching zone/sensor temperatures.
-
-            The temperature of each zone is reliably known (30C9 array), but the sensor
-            for each zone is not. In particular, the controller may be a sensor for a
-            zone, but unfortunately it does not announce its sensor temperatures.
-
-            In addition, there may be 'orphan' (e.g. from a neighbour) sensors
-            announcing temperatures with the same value.
-
-            This leaves only a process of exclusion as a means to determine which zone
-            uses the controller as a sensor.
-            """
-
-            def match_sensors(testable_sensors, zone_idx, zone_temp) -> list:
-                return [
-                    s
-                    for s in testable_sensors
-                    if s.temperature == zone_temp
-                    and (s.zone is None or s.zone.idx == zone_idx)
-                ]
-
-            def _testable_zones(changed_zones) -> dict:
-                return {
-                    z: t
-                    for z, t in changed_zones.items()
-                    if self.zone_by_idx[z].sensor is None
-                    # and t is not None  # done in changed_zones = {}
-                    and t not in [t2 for z2, t2 in changed_zones.items() if z2 != z]
-                }  # zones with unique (non-null) temps, and no sensor
-
-            assert isinstance(this.payload, list), "payload is not a list"
-
-            self._prev_30c9, prev = this, self._prev_30c9
-            if prev is None:
-                return
-
-            # TODO: use msgz/I, not RP
-            secs = self._msg_value(Code._1F09, key="remaining_seconds")
-            if secs is None or this.dtm > prev.dtm + td(seconds=secs + 5):
-                return  # can only compare against 30C9 pkt from the last cycle
-
-            _LOGGER.debug("System state (before): %s", self.schema)
-
-            changed_zones = {
-                z[SZ_ZONE_IDX]: z[SZ_TEMPERATURE]
-                for z in this.payload
-                if z not in prev.payload and z[SZ_TEMPERATURE] is not None
-            }  # zones with changed temps
-            _LOGGER.debug("Changed zones (from 30C9): %s", changed_zones)
-            if not changed_zones:
-                return  # ctl's 30C9 says no zones have changed temps during this cycle
-
-            testable_zones = _testable_zones(changed_zones)
-            _LOGGER.debug(
-                " - has unique/non-null temps (from 30C9) & no sensor (from state): %s",
-                testable_zones,
-            )
-            if not testable_zones:
-                return  # no testable zones
-
-            testable_sensors = [
-                d
-                for d in self._gwy.devices  # NOTE: *not* self.childs
-                if isinstance(d, Temperature)  # d.addr.type in DEVICE_HAS_ZONE_SENSOR
-                and d.ctl in (self.ctl, None)
-                and d.temperature is not None
-                and d._msgs[Code._30C9].dtm > prev.dtm  # changed during last cycle
-            ]
-
-            if _LOGGER.isEnabledFor(logging.DEBUG):
-                _LOGGER.debug(
-                    "Testable zones: %s (unique/non-null temps & sensorless)",
-                    testable_zones,
-                )
-                _LOGGER.debug(
-                    "Testable sensors: %s (non-null temps & parentless)",
-                    {d.id: d.temperature for d in testable_sensors},
-                )
-
-            if testable_sensors:  # the main matching algorithm...
-                for zone_idx, temp in testable_zones.items():
-                    # TODO: when sensors announce temp, ?also includes its parent zone
-                    matching_sensors = match_sensors(testable_sensors, zone_idx, temp)
-                    _LOGGER.debug("Testing zone %s, temp: %s", zone_idx, temp)
-                    _LOGGER.debug(
-                        " - matching sensor(s): %s (same temp & not from another zone)",
-                        [s.id for s in matching_sensors],
-                    )
-
-                    if len(matching_sensors) == 1:
-                        _LOGGER.debug(
-                            "   - matched sensor: %s (%s) to zone: %s (%s)",
-                            matching_sensors[0].id,
-                            matching_sensors[0].temperature,
-                            zone_idx,
-                            temp,
-                        )
-                        zone = self.zone_by_idx[zone_idx]
-                        self._gwy.get_device(
-                            matching_sensors[0].id, parent=zone, is_sensor=True
-                        )
-                    elif len(matching_sensors) == 0:
-                        _LOGGER.debug("   - no matching sensor (uses CTL?)")
-                    else:
-                        _LOGGER.debug("   - multiple sensors: %s", matching_sensors)
-
-                _LOGGER.debug("System state (after): %s", self.schema)
-
-            # now see if we can allocate the controller as a sensor...
-            if any(z for z in self.zones if z.sensor is self.ctl):
-                return  # the controller is already a sensor
-            if len([z for z in self.zones if z.sensor is None]) != 1:
-                return  # no single zone without a sensor
-
-            remaining_zones = _testable_zones(changed_zones)
-            if not remaining_zones:
-                return  # no testable zones
-
-            zone_idx, temp = list(remaining_zones.items())[0]
-            _LOGGER.debug("Testing (sole remaining) zone %s, temp: %s", zone_idx, temp)
-            # want to avoid complexity of z._temp
-            # zone = self.zone_by_idx[zone_idx]
-            # if zone._temp is None:
-            #     return  # TODO: should have a (not-None) temperature
-
-            matching_sensors = match_sensors(testable_sensors, zone_idx, temp)
-            _LOGGER.debug(
-                " - matching sensor(s): %s (excl. controller)",
-                [s.id for s in matching_sensors],
-            )
-
-            # can safely(?) assume this zone is using the CTL as a sensor...
-            if len(matching_sensors) == 0:
-                _LOGGER.debug("   - assumed sensor: %s (by exclusion)", self.ctl.id)
-                zone = self.zone_by_idx[zone_idx]
-                self._gwy.get_device(self.ctl.id, parent=zone, is_sensor=True)
-
-            _LOGGER.debug("System state (finally): %s", self.schema)
-
-        def eavesdrop_zone_sensors_new(this, *, prev=None) -> None:
             """Determine each zone's sensor by matching zone/sensor temperatures."""
 
             def _testable_zones(changed_zones) -> dict:
@@ -553,7 +414,7 @@ class MultiZone(SystemBase):  # 0005 (+/- 000C?)
             if secs is None or this.dtm > prev.dtm + td(seconds=secs + 5):
                 return  # can only compare against 30C9 pkt from the last cycle
 
-            _LOGGER.debug("System state (before): %s", self.schema)
+            # _LOGGER.warning("System state (before): %s", self.schema)
 
             changed_zones = {
                 z[SZ_ZONE_IDX]: z[SZ_TEMPERATURE]
@@ -589,7 +450,7 @@ class MultiZone(SystemBase):  # 0005 (+/- 000C?)
                 zone = self.zone_by_idx[zone_idx]
                 self._gwy.get_device(sensor.id, parent=zone, is_sensor=True)
 
-            _LOGGER.debug("System state (after): %s", self.schema)
+            # _LOGGER.warning("System state (after): %s", self.schema)
 
             # now see if we can allocate the controller as a sensor...
             if any(z for z in self.zones if z.sensor is self.ctl):
@@ -606,7 +467,7 @@ class MultiZone(SystemBase):  # 0005 (+/- 000C?)
                 zone = self.zone_by_idx[zone_idx]
                 self._gwy.get_device(self.ctl.id, parent=zone, is_sensor=True)
 
-            _LOGGER.debug("System state (finally): %s", self.schema)
+            # _LOGGER.warning("System state (finally): %s", self.schema)
 
         def handle_msg_by_zone_idx(zone_idx: str, msg):
             if zone := self.zone_by_idx.get(zone_idx):
@@ -739,12 +600,10 @@ class ScheduleSync(SystemBase):  # 0006 (+/- 0404?)
         self.zone_lock = Lock()  # used to stop concurrent get_schedules
         self.zone_lock_idx = None
 
-    def _setup_discovery_tasks(self) -> None:
-        super()._setup_discovery_tasks()
+    def _setup_discovery_cmds(self) -> None:
+        super()._setup_discovery_cmds()
 
-        self._add_discovery_task(
-            _mk_cmd(RQ, Code._0006, "00", self.id), 60 * 5, delay=5
-        )
+        self._add_discovery_cmd(_mk_cmd(RQ, Code._0006, "00", self.id), 60 * 5, delay=5)
 
     def _handle_msg(self, msg: Message) -> None:  # NOTE: active
         """Periodically retreive the latest global change counter."""
@@ -831,10 +690,10 @@ class ScheduleSync(SystemBase):  # 0006 (+/- 0404?)
 
 
 class Language(SystemBase):  # 0100
-    def _setup_discovery_tasks(self) -> None:
-        super()._setup_discovery_tasks()
+    def _setup_discovery_cmds(self) -> None:
+        super()._setup_discovery_cmds()
 
-        self._add_discovery_task(
+        self._add_discovery_cmd(
             Command.get_system_language(self.id), 60 * 60 * 24, delay=60 * 15
         )
 
@@ -863,10 +722,10 @@ class Logbook(SystemBase):  # 0418
         self._faultlog: FaultLog = None  # type: ignore[assignment]
         self._faultlog_outdated: bool = True
 
-    def _setup_discovery_tasks(self) -> None:
-        super()._setup_discovery_tasks()
+    def _setup_discovery_cmds(self) -> None:
+        super()._setup_discovery_cmds()
 
-        self._add_discovery_task(
+        self._add_discovery_cmd(
             Command.get_system_log_entry(self.ctl.id, 0), 60 * 5, delay=5
         )
         # self._gwy.add_task(self._loop.create_task(self.get_faultlog()))
@@ -958,15 +817,15 @@ class StoredHw(SystemBase):  # 10A0, 1260, 1F41
         super().__init__(*args, **kwargs)
         self._dhw: DhwZone = None  # type: ignore[assignment]
 
-    def _setup_discovery_tasks(self) -> None:
-        super()._setup_discovery_tasks()
+    def _setup_discovery_cmds(self) -> None:
+        super()._setup_discovery_cmds()
 
         for payload in (
             f"00{DEV_ROLE_MAP.DHW}",  # dhw_sensor
             # f"00{DEV_ROLE_MAP.HTG}",  # hotwater_valve
             # f"01{DEV_ROLE_MAP.HTG}",  # heating_valve
         ):
-            self._add_discovery_task(
+            self._add_discovery_cmd(
                 _mk_cmd(RQ, Code._000C, payload, self.id), 60 * 60 * 24, delay=0
             )
 
@@ -1061,10 +920,10 @@ class StoredHw(SystemBase):  # 10A0, 1260, 1F41
 
 
 class SysMode(SystemBase):  # 2E04
-    def _setup_discovery_tasks(self) -> None:
-        super()._setup_discovery_tasks()
+    def _setup_discovery_cmds(self) -> None:
+        super()._setup_discovery_cmds()
 
-        self._add_discovery_task(Command.get_system_mode(self.id), 60 * 5, delay=5)
+        self._add_discovery_cmd(Command.get_system_mode(self.id), 60 * 5, delay=5)
 
     @property
     def system_mode(self) -> Optional[dict]:  # 2E04
@@ -1092,10 +951,10 @@ class SysMode(SystemBase):  # 2E04
 
 
 class Datetime(SystemBase):  # 313F
-    def _setup_discovery_tasks(self) -> None:
-        super()._setup_discovery_tasks()
+    def _setup_discovery_cmds(self) -> None:
+        super()._setup_discovery_cmds()
 
-        self._add_discovery_task(Command.get_system_time(self.id), 60 * 60, delay=0)
+        self._add_discovery_cmd(Command.get_system_time(self.id), 60 * 60, delay=0)
 
     def _handle_msg(self, msg: Message) -> None:
         super()._handle_msg(msg)
@@ -1231,8 +1090,8 @@ class Hometronics(System):
     _SLUG: str = SYS_KLASS.SYS
 
     #
-    # def _setup_discovery_tasks(self) -> None:
-    #     # super()._setup_discovery_tasks()
+    # def _setup_discovery_cmds(self) -> None:
+    #     # super()._setup_discovery_cmds()
 
     #     # will RP to: 0005/configured_zones_alt, but not: configured_zones
     #     # will RP to: 0004
