@@ -42,6 +42,7 @@ from ..entity_base import Child, Entity, Parent, class_by_attr
 from ..helpers import shrink
 from ..protocol import Address, Command, CorruptStateError, Message
 from ..protocol.command import _mk_cmd
+from ..protocol.const import SZ_PAYLOAD
 from ..schemas import (
     SCH_TCS_DHW,
     SCH_TCS_ZONES_ZON,
@@ -126,7 +127,7 @@ class ZoneBase(Child, Parent, Entity):
         return self.idx < other.idx
 
     def _make_cmd(self, code, **kwargs) -> None:  # skipcq: PYL-W0221
-        payload = kwargs.pop("payload", f"{self.idx}00")
+        payload = kwargs.pop(SZ_PAYLOAD, f"{self.idx}00")
         super()._make_cmd(code, self.ctl.id, payload=payload, **kwargs)
 
     @property
@@ -143,13 +144,7 @@ class ZoneSchedule:  # 0404
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
 
-        self._schedule = Schedule(self)
-
-    def OUT_discover(self) -> None:
-        super()._discover()
-
-        if False:
-            self._loop.create_task(self.get_schedule())  # 0404
+        self._schedule = Schedule(self)  # ? add to discovery
 
     def _handle_msg(self, msg: Message) -> None:
         super()._handle_msg(msg)
@@ -202,22 +197,22 @@ class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
         self._dhw_valve: BdrSwitch = None  # type: ignore[assignment]
         self._htg_valve: BdrSwitch = None  # type: ignore[assignment]
 
-    def _setup_discovery_tasks(self) -> None:
-        # super()._setup_discovery_tasks()
+    def _setup_discovery_cmds(self) -> None:
+        # super()._setup_discovery_cmds()
 
         for payload in (
             f"00{DEV_ROLE_MAP.DHW}",
             f"00{DEV_ROLE_MAP.HTG}",
             f"01{DEV_ROLE_MAP.HTG}",
         ):
-            self._add_discovery_task(
+            self._add_discovery_cmd(
                 _mk_cmd(RQ, Code._000C, payload, self.ctl.id), 60 * 60 * 24
             )
 
-        self._add_discovery_task(Command.get_dhw_params(self.ctl.id), 60 * 60 * 6)
+        self._add_discovery_cmd(Command.get_dhw_params(self.ctl.id), 60 * 60 * 6)
 
-        self._add_discovery_task(Command.get_dhw_mode(self.ctl.id), 60 * 5)
-        self._add_discovery_task(Command.get_dhw_temp(self.ctl.id), 60 * 15)
+        self._add_discovery_cmd(Command.get_dhw_mode(self.ctl.id), 60 * 5)
+        self._add_discovery_cmd(Command.get_dhw_temp(self.ctl.id), 60 * 15)
 
     def _handle_msg(self, msg: Message) -> None:
         def eavesdrop_dhw_sensor(this, *, prev=None) -> None:
@@ -496,7 +491,7 @@ class Zone(ZoneSchedule, ZoneBase):
             _LOGGER.debug("Promoted a Zone: %s (%s)", self.id, self.__class__)
 
             # TODO: broken fixme
-            # self._gwy._loop.call_soon(self._setup_discovery_tasks)  # TODO: check this
+            # self._gwy._loop.call_soon(self._setup_discovery_cmds)  # TODO: check this
 
         # if schema.get(SZ_CLASS) == ZON_ROLE_MAP[ZON_ROLE.ACT]:
         #     schema.pop(SZ_CLASS)
@@ -511,49 +506,49 @@ class Zone(ZoneSchedule, ZoneBase):
         for dev_id in schema.get(SZ_ACTUATORS, []):
             self._gwy.get_device(dev_id, parent=self)
 
-    def _setup_discovery_tasks(self) -> None:
-        # super()._setup_discovery_tasks()
+    def _setup_discovery_cmds(self) -> None:
+        # super()._setup_discovery_cmds()
 
         for dev_role in (self._ROLE_ACTUATORS, DEV_ROLE_MAP.SEN):
-            self._add_discovery_task(
+            self._add_discovery_cmd(
                 _mk_cmd(RQ, Code._000C, f"{self.idx}{dev_role}", self.ctl.id),
                 60 * 60 * 24,
                 delay=0.5,
             )
 
-        self._add_discovery_task(
+        self._add_discovery_cmd(
             Command.get_zone_config(self.ctl.id, self.idx), 60 * 60 * 6, delay=30
         )  # td should be > long sync_cycle duration (> 1hr)
-        self._add_discovery_task(
+        self._add_discovery_cmd(
             Command.get_zone_name(self.ctl.id, self.idx), 60 * 60 * 6, delay=30
         )
 
-        self._add_discovery_task(  # 2349 instead of 2309
+        self._add_discovery_cmd(  # 2349 instead of 2309
             Command.get_zone_mode(self.ctl.id, self.idx), 60 * 5, delay=30
         )
-        self._add_discovery_task(  # 30C9
+        self._add_discovery_cmd(  # 30C9
             Command.get_zone_temp(self.ctl.id, self.idx), 60 * 5, delay=0
         )  # td should be > sync_cycle duration,?delay in hope of picking up cycle
-        self._add_discovery_task(
+        self._add_discovery_cmd(
             Command.get_zone_window_state(self.ctl.id, self.idx), 60 * 15, delay=60 * 5
         )  # longer dt as low yield (factory duration is 30 min): prefer eavesdropping
 
-    def _add_discovery_task(
+    def _add_discovery_cmd(
         self, cmd, interval, *, delay: float = 0, timeout: float = None
     ):
         """Schedule a command to run periodically."""
-        super()._add_discovery_task(cmd, interval, delay=delay, timeout=timeout)
+        super()._add_discovery_cmd(cmd, interval, delay=delay, timeout=timeout)
 
         if cmd.code != Code._000C:  # or cmd._ctx == f"{self.idx}{ZON_ROLE_MAP.SEN}":
             return
 
-        if [t for t in self._disc_tasks if t[-2:] in ZON_ROLE_MAP.HEAT_ZONES] and (
-            self._disc_tasks.pop(f"{self.idx}{ZON_ROLE_MAP.ACT}", None)
+        if [t for t in self._discovery_cmds if t[-2:] in ZON_ROLE_MAP.HEAT_ZONES] and (
+            self._discovery_cmds.pop(f"{self.idx}{ZON_ROLE_MAP.ACT}", None)
         ):
             _LOGGER.warning(f"cmd({cmd}): inferior header removed from discovery")
 
-        if self._disc_tasks.get(f"{self.idx}{ZON_ROLE_MAP.VAL}") and (
-            self._disc_tasks[f"{self.idx}{ZON_ROLE_MAP.ELE}"]
+        if self._discovery_cmds.get(f"{self.idx}{ZON_ROLE_MAP.VAL}") and (
+            self._discovery_cmds[f"{self.idx}{ZON_ROLE_MAP.ELE}"]
         ):
             _LOGGER.warning(f"cmd({cmd}): inferior header removed from discovery")
 
@@ -576,9 +571,8 @@ class Zone(ZoneSchedule, ZoneBase):
                 ), self._SLUG
 
                 if self._SLUG is None:
-                    self._update_schema(
-                        {SZ_CLASS: ZON_ROLE.ELE}
-                    )  # might eventually be: ZON_ROLE.VAL
+                    # this might eventually be: ZON_ROLE.VAL
+                    self._update_schema(**{SZ_CLASS: ZON_ROLE_MAP[ZON_ROLE.ELE]})
 
             elif this.code == Code._3150:  # TODO: and this.verb in (I_, RP)?
                 # MIX/ELE don't 3150
@@ -590,11 +584,11 @@ class Zone(ZoneSchedule, ZoneBase):
                 ), self._SLUG
 
                 if isinstance(this.src, TrvActuator):
-                    self._update_schema({SZ_CLASS: ZON_ROLE.RAD})
+                    self._update_schema(**{SZ_CLASS: ZON_ROLE_MAP[ZON_ROLE.RAD]})
                 elif isinstance(this.src, BdrSwitch):
-                    self._update_schema({SZ_CLASS: ZON_ROLE.VAL})
+                    self._update_schema(**{SZ_CLASS: ZON_ROLE_MAP[ZON_ROLE.VAL]})
                 elif isinstance(this.src, UfhController):
-                    self._update_schema({SZ_CLASS: ZON_ROLE.UFH})
+                    self._update_schema(**{SZ_CLASS: ZON_ROLE_MAP[ZON_ROLE.UFH]})
 
             assert (
                 msg.src is self.ctl or msg.src.type == DEV_TYPE_MAP.UFC
@@ -789,7 +783,7 @@ class EleZone(Zone):  # BDR91A/T  # TODO: 0008/0009/3150
         super()._handle_msg(msg)
 
         # if msg.code == Code._0008:  # ZON zones are ELE zones that also call for heat
-        #     self._update_schema({SZ_CLASS: ZON_ROLE.VAL})
+        #     self._update_schema(**{SZ_CLASS: ZON_ROLE_MAP[ZON_ROLE.VAL]})
         if msg.code == Code._3150:
             raise TypeError("WHAT 1")
         elif msg.code == Code._3EF0:
@@ -825,10 +819,10 @@ class MixZone(Zone):  # HM80  # TODO: 0008/0009/3150
     _SLUG: str = ZON_ROLE.MIX
     _ROLE_ACTUATORS: str = DEV_ROLE_MAP.MIX
 
-    def _setup_discovery_tasks(self) -> None:
-        super()._setup_discovery_tasks()
+    def _setup_discovery_cmds(self) -> None:
+        super()._setup_discovery_cmds()
 
-        self._add_discovery_task(
+        self._add_discovery_cmd(
             Command.get_mix_valve_params(self.ctl.id, self.idx), 60 * 60 * 6
         )
 
